@@ -1,22 +1,22 @@
-// =====================================================================
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// hics206.cxx
+// This file is part of flmsg
 //
-// Author: Dave Freese, W1HKJ
-// Copyright: 2010
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  It is
-// copyright under the GNU General Public License.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with the program; if not, write to the Free Software
-// Foundation, Inc.
-// 59 Temple Place, Suite 330
-// Boston, MA  02111-1307 USA
-//
-// =====================================================================
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <stdlib.h>
 #include <iostream>
@@ -254,6 +254,110 @@ void h206_clear_fields()
 
 }
 
+bool check_hics206fields()
+{
+	if (h206_name != h206_txt_name->value())
+		return true;
+	if (h206_period != h206_txt_op_period->value())
+		return true;
+	if (h206_date_prepared != h206_txt_date_prepared->value())
+		return true;
+	if (h206_time_prepared != h206_txt_time_prepared->value())
+		return true;
+
+	if (h206_location != h206_txt_location->value())
+		return true;
+	if (h206_loc_contact != h206_txt_loc_contact->value())
+		return true;
+	if (h206_team_leader != h206_txt_team_leader->value())
+		return true;
+	if (h206_team_contact != h206_txt_team_contact->value())
+		return true;
+	if (h206_md_do != h206_txt_md_do->value())
+		return true;
+	if (h206_litters != h206_txt_litters->value())
+		return true;
+	if (h206_pa_np != h206_txt_pa_np->value())
+		return true;
+	if (h206_portable != h206_txt_portable->value())
+		return true;
+	if (h206_rn_lpn != h206_txt_rn_lpn->value())
+		return true;
+	if (h206_transport != h206_txt_transport->value())
+		return true;
+	if (h206_tech_cn != h206_txt_tech_cn->value())
+		return true;
+	if (h206_wheelchairs != h206_txt_wheelchairs->value())
+		return true;
+	if (h206_team_other != h206_txt_team_other->value())
+		return true;
+	if (h206_transport_other != h206_txt_transport_other->value())
+		return true;
+	if (h206_meds_1 != h206_txt_meds_1->value())
+		return true;
+	if (h206_supp_1 != h206_txt_supp_1->value())
+		return true;
+	if (h206_meds_2 != h206_txt_meds_2->value())
+		return true;
+	if (h206_supp_2 != h206_txt_supp_2->value())
+		return true;
+	if (h206_meds_3 != h206_txt_meds_3->value())
+		return true;
+	if (h206_supp_3 != h206_txt_supp_3->value())
+		return true;
+	if (h206_meds_4 != h206_txt_meds_4->value())
+		return true;
+	if (h206_supp_4 != h206_txt_supp_4->value())
+		return true;
+	if (h206_meds_5 != h206_txt_meds_5->value())
+		return true;
+	if (h206_supp_5 != h206_txt_supp_5->value())
+		return true;
+
+	if (h206_special_instructions != h206_txt_special_instructions->buffer()->text())
+		return true;
+
+	if (h206_preparer != h206_txt_preparer->value())
+		return true;
+	if (h206_facility != h206_txt_facility->value())
+		return true;
+
+	if (h206_site_1 != h206_txt_site_1->value())
+		return true;
+	if (h206_address_1 != h206_txt_address_1->value())
+		return true;
+	if (h206_phone_1 != h206_txt_phone_1->value())
+		return true;
+	if (h206_spec_care_1 != h206_txt_spec_care_1->value())
+		return true;
+	if (h206_site_2 != h206_txt_site_2->value())
+		return true;
+	if (h206_address_2 != h206_txt_address_2->value())
+		return true;
+	if (h206_phone_2 != h206_txt_phone_2->value())
+		return true;
+	if (h206_spec_care_2 != h206_txt_spec_care_2->value())
+		return true;
+	if (h206_site_3 != h206_txt_site_3->value())
+		return true;
+	if (h206_address_3 != h206_txt_address_3->value())
+		return true;
+	if (h206_phone_3 != h206_txt_phone_3->value())
+		return true;
+	if (h206_spec_care_3 != h206_txt_spec_care_3->value())
+		return true;
+	if (h206_site_4 != h206_txt_site_4->value())
+		return true;
+	if (h206_address_4 != h206_txt_address_4->value())
+		return true;
+	if (h206_phone_4 != h206_txt_phone_4->value())
+		return true;
+	if (h206_spec_care_4 != h206_txt_spec_care_4->value())
+		return true;
+
+	return false;
+}
+
 void h206_update_fields()
 {
 	h206_name = h206_txt_name->value();
@@ -265,7 +369,6 @@ void h206_update_fields()
 	h206_loc_contact = h206_txt_loc_contact->value();
 	h206_team_leader = h206_txt_team_leader->value();
 	h206_team_contact = h206_txt_team_contact->value();
-
 	h206_md_do = h206_txt_md_do->value();
 	h206_litters = h206_txt_litters->value();
 	h206_pa_np = h206_txt_pa_np->value();
@@ -308,7 +411,6 @@ void h206_update_fields()
 	h206_address_4 = h206_txt_address_4->value();
 	h206_phone_4 = h206_txt_phone_4->value();
 	h206_spec_care_4 = h206_txt_spec_care_4->value();
-
 }
 
 void h206_update_form()
@@ -427,63 +529,64 @@ void h206_clear__form()
 
 }
 
-void h206_make_buff()
+void h206_make_buff(bool compress = false)
 {
-	h206_update_fields();
+	string mbuff;
+	mbuff.assign( lineout( h206_tag_name, h206_name ) );
+	mbuff.append( lineout( h206_tag_period, h206_period ) );
+	mbuff.append( lineout( h206_tag_date_prepared, h206_date_prepared ) );
+	mbuff.append( lineout( h206_tag_time_prepared, h206_time_prepared ) );
 
-	h206_buff.append( lineout( h206_tag_name, h206_name ) );
-	h206_buff.append( lineout( h206_tag_period, h206_period ) );
-	h206_buff.append( lineout( h206_tag_date_prepared, h206_date_prepared ) );
-	h206_buff.append( lineout( h206_tag_time_prepared, h206_time_prepared ) );
+	mbuff.append( lineout( h206_tag_location, h206_location));
+	mbuff.append( lineout( h206_tag_loc_contact, h206_loc_contact));
+	mbuff.append( lineout( h206_tag_team_leader, h206_team_leader));
+	mbuff.append( lineout( h206_tag_team_contact, h206_team_contact));
 
-	h206_buff.append( lineout( h206_tag_location, h206_location));
-	h206_buff.append( lineout( h206_tag_loc_contact, h206_loc_contact));
-	h206_buff.append( lineout( h206_tag_team_leader, h206_team_leader));
-	h206_buff.append( lineout( h206_tag_team_contact, h206_team_contact));
+	mbuff.append( lineout( h206_tag_md_do, h206_md_do));
+	mbuff.append( lineout( h206_tag_litters, h206_litters));
+	mbuff.append( lineout( h206_tag_pa_np, h206_pa_np));
+	mbuff.append( lineout( h206_tag_portable, h206_portable));
+	mbuff.append( lineout( h206_tag_rn_lpn, h206_rn_lpn));
+	mbuff.append( lineout( h206_tag_transport, h206_transport));
+	mbuff.append( lineout( h206_tag_tech_cn, h206_tech_cn));
+	mbuff.append( lineout( h206_tag_wheelchairs, h206_wheelchairs));
+	mbuff.append( lineout( h206_tag_team_other, h206_team_other));
+	mbuff.append( lineout( h206_tag_transport_other, h206_transport_other));
+	mbuff.append( lineout( h206_tag_meds_1, h206_meds_1));
+	mbuff.append( lineout( h206_tag_supp_1, h206_supp_1));
+	mbuff.append( lineout( h206_tag_meds_2, h206_meds_2));
+	mbuff.append( lineout( h206_tag_supp_2, h206_supp_2));
+	mbuff.append( lineout( h206_tag_meds_3, h206_meds_3));
+	mbuff.append( lineout( h206_tag_supp_3, h206_supp_3));
+	mbuff.append( lineout( h206_tag_meds_4, h206_meds_4));
+	mbuff.append( lineout( h206_tag_supp_4, h206_supp_4));
+	mbuff.append( lineout( h206_tag_meds_5, h206_meds_5));
+	mbuff.append( lineout( h206_tag_supp_5, h206_supp_5));
 
-	h206_buff.append( lineout( h206_tag_md_do, h206_md_do));
-	h206_buff.append( lineout( h206_tag_litters, h206_litters));
-	h206_buff.append( lineout( h206_tag_pa_np, h206_pa_np));
-	h206_buff.append( lineout( h206_tag_portable, h206_portable));
-	h206_buff.append( lineout( h206_tag_rn_lpn, h206_rn_lpn));
-	h206_buff.append( lineout( h206_tag_transport, h206_transport));
-	h206_buff.append( lineout( h206_tag_tech_cn, h206_tech_cn));
-	h206_buff.append( lineout( h206_tag_wheelchairs, h206_wheelchairs));
-	h206_buff.append( lineout( h206_tag_team_other, h206_team_other));
-	h206_buff.append( lineout( h206_tag_transport_other, h206_transport_other));
-	h206_buff.append( lineout( h206_tag_meds_1, h206_meds_1));
-	h206_buff.append( lineout( h206_tag_supp_1, h206_supp_1));
-	h206_buff.append( lineout( h206_tag_meds_2, h206_meds_2));
-	h206_buff.append( lineout( h206_tag_supp_2, h206_supp_2));
-	h206_buff.append( lineout( h206_tag_meds_3, h206_meds_3));
-	h206_buff.append( lineout( h206_tag_supp_3, h206_supp_3));
-	h206_buff.append( lineout( h206_tag_meds_4, h206_meds_4));
-	h206_buff.append( lineout( h206_tag_supp_4, h206_supp_4));
-	h206_buff.append( lineout( h206_tag_meds_5, h206_meds_5));
-	h206_buff.append( lineout( h206_tag_supp_5, h206_supp_5));
+	mbuff.append( lineout( h206_tag_special_instructions, h206_special_instructions ) );
 
-	h206_buff.append( lineout( h206_tag_special_instructions, h206_special_instructions ) );
+	mbuff.append( lineout( h206_tag_preparer, h206_preparer ) );
+	mbuff.append( lineout( h206_tag_facility, h206_facility ) );
 
-	h206_buff.append( lineout( h206_tag_preparer, h206_preparer ) );
-	h206_buff.append( lineout( h206_tag_facility, h206_facility ) );
-
-	h206_buff.append( lineout( h206_tag_site_1, h206_site_1 ) );
-	h206_buff.append( lineout( h206_tag_address_1, h206_address_1 ) );
-	h206_buff.append( lineout( h206_tag_phone_1, h206_phone_1 ) );
-	h206_buff.append( lineout( h206_tag_spec_care_1, h206_spec_care_1 ) );
-	h206_buff.append( lineout( h206_tag_site_2, h206_site_2 ) );
-	h206_buff.append( lineout( h206_tag_address_2, h206_address_2 ) );
-	h206_buff.append( lineout( h206_tag_phone_2, h206_phone_2 ) );
-	h206_buff.append( lineout( h206_tag_spec_care_2, h206_spec_care_2 ) );
-	h206_buff.append( lineout( h206_tag_site_3, h206_site_3 ) );
-	h206_buff.append( lineout( h206_tag_address_3, h206_address_3 ) );
-	h206_buff.append( lineout( h206_tag_phone_3, h206_phone_3 ) );
-	h206_buff.append( lineout( h206_tag_spec_care_3, h206_spec_care_3 ) );
-	h206_buff.append( lineout( h206_tag_site_4, h206_site_4 ) );
-	h206_buff.append( lineout( h206_tag_address_4, h206_address_4 ) );
-	h206_buff.append( lineout( h206_tag_phone_4, h206_phone_4 ) );
-	h206_buff.append( lineout( h206_tag_spec_care_4, h206_spec_care_4 ) );
+	mbuff.append( lineout( h206_tag_site_1, h206_site_1 ) );
+	mbuff.append( lineout( h206_tag_address_1, h206_address_1 ) );
+	mbuff.append( lineout( h206_tag_phone_1, h206_phone_1 ) );
+	mbuff.append( lineout( h206_tag_spec_care_1, h206_spec_care_1 ) );
+	mbuff.append( lineout( h206_tag_site_2, h206_site_2 ) );
+	mbuff.append( lineout( h206_tag_address_2, h206_address_2 ) );
+	mbuff.append( lineout( h206_tag_phone_2, h206_phone_2 ) );
+	mbuff.append( lineout( h206_tag_spec_care_2, h206_spec_care_2 ) );
+	mbuff.append( lineout( h206_tag_site_3, h206_site_3 ) );
+	mbuff.append( lineout( h206_tag_address_3, h206_address_3 ) );
+	mbuff.append( lineout( h206_tag_phone_3, h206_phone_3 ) );
+	mbuff.append( lineout( h206_tag_spec_care_3, h206_spec_care_3 ) );
+	mbuff.append( lineout( h206_tag_site_4, h206_site_4 ) );
+	mbuff.append( lineout( h206_tag_address_4, h206_address_4 ) );
+	mbuff.append( lineout( h206_tag_phone_4, h206_phone_4 ) );
+	mbuff.append( lineout( h206_tag_spec_care_4, h206_spec_care_4 ) );
 
+	if (compress) compress_maybe(mbuff);
+	h206_buff.append(mbuff);
 }
 
 void h206_read_buffer(string data)
@@ -549,6 +652,12 @@ void h206_read_buffer(string data)
 
 void h206_cb_new()
 {
+	if (check_hics206fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 1) {
+			update_header(CHANGED);
+			h206_cb_save();
+		}
+	}
 	h206_clear__form();
 	clear_header();
 	h206_def_filename = ICS_msg_dir;
@@ -577,10 +686,34 @@ void h206_cb_wrap_import(string wrapfilename, string inpbuffer)
 	h206_using206_template = false;
 }
 
+int eval_h206_fsize()
+{
+	Ccrc16 chksum;
+	evalstr.assign("[WRAP:beg][WRAP:lf][WRAP:fn ");
+	evalstr.append(h206_base_filename).append("]");
+	h206_update_fields();
+	update_header(FROM);
+	evalstr.append(header("<hics206>"));
+	h206_buff.clear();
+	h206_make_buff(true);
+	if (h206_buff.empty()) return 0;
+	compress_maybe( h206_buff );
+	evalstr.append( h206_buff );
+	evalstr.append("[WRAP:chksum ").append(chksum.scrc16(evalstr)).append("][WRAP:end]");
+	return evalstr.length();
+}
+
 void h206_cb_wrap_export()
 {
+	if (check_hics206fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	h206_update_fields();
+
 	if (h206_base_filename == "new"HF206_EXT || h206_base_filename == "default"HF206_EXT)
-		h206_cb_save_as();
+		if (!h206_cb_save_as()) return;
 
 	string wrapfilename = WRAP_send_dir;
 	wrapfilename.append(h206_base_filename);
@@ -592,26 +725,37 @@ void h206_cb_wrap_export()
 	if (p) {
 		string pext = fl_filename_ext(p);
 		wrapfilename = p;
-		update_header(true);
-		h206_buff.assign(header("<hics206>", true, true));
-		h206_make_buff();
+		update_header(FROM);
+		h206_buff.assign(header("<hics206>"));
+		h206_make_buff(true);
 		export_wrapfile(h206_base_filename, wrapfilename, h206_buff, pext != ".wrap");
+
+		h206_buff.assign(header("<hics206>"));
+		h206_make_buff(false);
+		h206_write(h206_def_filename);
 	}
 }
 
 void h206_cb_wrap_autosend()
 {
-	if (h206_base_filename == "new"HF206_EXT || 
-		h206_base_filename == "default"HF206_EXT ||
-		h206_using206_template == true)
-		h206_cb_save_as();
+	if (check_hics206fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	h206_update_fields();
 
-	string wrapfilename = WRAP_auto_dir;
-	wrapfilename.append("wrap_auto_file");
-	update_header(true);
-	h206_buff.assign(header("<hics206>", true, true));
-	h206_make_buff();
-	export_wrapfile(h206_base_filename, wrapfilename, h206_buff, false);
+	if (h206_base_filename == "new"HF206_EXT || h206_base_filename == "default"HF206_EXT)
+		if (!h206_cb_save_as()) return;
+
+	update_header(FROM);
+	h206_buff.assign(header("<hics206>"));
+	h206_make_buff(true);
+	xfr_via_socket(h206_base_filename, h206_buff);
+
+	h206_buff.assign(header("<hics206>"));
+	h206_make_buff(false);
+	h206_write(h206_def_filename);
 }
 
 void h206_cb_load_template()
@@ -642,7 +786,10 @@ void h206_cb_save_template()
 			"Template file\t*"HT206_EXT,
 			h206_def_filename.c_str());
 	if (p) {
-		clear_header();
+		update_header(CHANGED);
+		h206_update_fields();
+		h206_buff.assign(header("<hics206>"));
+		h206_make_buff();
 		h206_write(p);
 	}
 }
@@ -660,6 +807,10 @@ void h206_cb_save_as_template()
 		if (strlen(pext) == 0) h206_def_template_name.append(HT206_EXT);
 		remove_spaces_from_filename(h206_def_template_name);
 		clear_header();
+		update_header(CHANGED);
+		h206_update_fields();
+		h206_buff.assign(header("<hics206>"));
+		h206_make_buff();
 		h206_write(h206_def_template_name);
 		show_filename(h206_def_template_name);
 		h206_using206_template = true;
@@ -683,14 +834,12 @@ void h206_write(string s)
 {
 	FILE *file206 = fopen(s.c_str(), "w");
 	if (!file206) return;
-	update_header();
-	h206_buff.assign(save_header("<hics206>"));
-	h206_make_buff();
+
 	fwrite(h206_buff.c_str(), h206_buff.length(), 1, file206);
 	fclose(file206);
 }
 
-void h206_cb_save_as()
+bool h206_cb_save_as()
 {
 	const char *p;
 	string newfilename;
@@ -705,8 +854,10 @@ void h206_cb_save_as()
 
 	p = FSEL::saveas(_("Save data file"), "HICS-206\t*"HF206_EXT,
 					newfilename.c_str());
-	if (!p) return;
-	if (strlen(p) == 0) return;
+
+	if (!p) return false;
+	if (strlen(p) == 0) return false;
+
 	if (progStatus.sernbr_fname) {
 		string haystack = p;
 		if (haystack.find(newfilename) != string::npos) {
@@ -725,13 +876,14 @@ void h206_cb_save_as()
 	if (strlen(pext) == 0) h206_def_filename.append(HF206_EXT);
 
 	remove_spaces_from_filename(h206_def_filename);
-	clear_header();
-	update_header();
-	h206_buff.assign(save_header("<hics206>"));
+	h206_update_fields();
+	update_header(NEW);
+	h206_buff.assign(header("<hics206>"));
 	h206_write(h206_def_filename);
 
 	h206_using206_template = false;
 	show_filename(h206_def_filename);
+	return true;
 }
 
 void h206_cb_save()
@@ -742,8 +894,9 @@ void h206_cb_save()
 		h206_cb_save_as();
 		return;
 	}
-	update_header();
-	h206_buff.assign(save_header("<hics206>"));
+	if (check_hics206fields()) update_header(CHANGED);
+	h206_buff.assign(header("<hics206>"));
+	h206_make_buff();
 	h206_write(h206_def_filename);
 	h206_using206_template = false;
 }
