@@ -1,22 +1,22 @@
-// =====================================================================
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// iaru.cxx
+// This file is part of flmsg
 //
-// Author: Dave Freese, W1HKJ
-// Copyright: 2010
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  It is
-// copyright under the GNU General Public License.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with the program; if not, write to the Free Software
-// Foundation, Inc.
-// 59 Temple Place, Suite 330
-// Boston, MA  02111-1307 USA
-//
-// =====================================================================
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <stdlib.h>
 #include <iostream>
@@ -201,24 +201,54 @@ static string numeric(int n)
 void iaru_set_choices() {
 	iaru_sel_prec->clear();
 	iaru_sel_prec->add(iaru_prec_items);
-	iaru_sel_prec->value(0);
+	iaru_sel_prec->index(0);
+}
+
+bool iaru_check_fields()
+{
+	for (int i = 0; i < iaru_num_fields; i++) {
+		if (iaru_fields[i].w_type == 'd') {
+			if (iaru_fields[i].f_data != ((Fl_DateInput *)(*iaru_fields[i].w))->value())
+				return true;
+		} else if (iaru_fields[i].w_type == 't') {
+			if (iaru_fields[i].f_data != ((Fl_Input2 *)(*iaru_fields[i].w))->value())
+				return true;
+		} else if (iaru_fields[i].w_type == 's') {
+			int choice = ((Fl_ListBox *)(*iaru_fields[i].w))->index();
+			if (choice >= 0) {
+				if (iaru_fields[i].f_data != numeric(choice))
+					return true;
+			}
+		} else if (iaru_fields[i].w_type == 'e') {
+			if (iaru_fields[i].f_data != ((FTextEdit *)(*iaru_fields[i].w))->buffer()->text())
+				return true;
+		} else if (iaru_fields[i].w_type == 'b') {
+			string val = ((Fl_Button *)(*iaru_fields[i].w))->value() ? "T" : "F";
+			if (iaru_fields[i].f_data != val)
+				return true;
+		}
+	}
+	return false;
 }
 
 void iaru_update_fields()
 {
 	for (int i = 0; i < iaru_num_fields; i++) {
-		if (iaru_fields[i].w_type == 'd')
+		if (iaru_fields[i].w_type == 'd') {
 			iaru_fields[i].f_data = ((Fl_DateInput *)(*iaru_fields[i].w))->value();
-		else if (iaru_fields[i].w_type == 't')
+		} else if (iaru_fields[i].w_type == 't') {
 			iaru_fields[i].f_data = ((Fl_Input2 *)(*iaru_fields[i].w))->value();
-		else if (iaru_fields[i].w_type == 's') {
-			int choice = ((Fl_Choice *)(*iaru_fields[i].w))->value();
-			if (choice >= 0) 
+		} else if (iaru_fields[i].w_type == 's') {
+			int choice = ((Fl_ListBox *)(*iaru_fields[i].w))->index();
+			if (choice >= 0) {
 				iaru_fields[i].f_data = numeric(choice);
-		} else if (iaru_fields[i].w_type == 'e')
+			}
+		} else if (iaru_fields[i].w_type == 'e') {
 			iaru_fields[i].f_data = ((FTextEdit *)(*iaru_fields[i].w))->buffer()->text();
-		else if (iaru_fields[i].w_type == 'b')
-			iaru_fields[i].f_data = ((Fl_Button *)(*iaru_fields[i].w))->value() ? "T" : "F";
+		} else if (iaru_fields[i].w_type == 'b') {
+			string val = ((Fl_Button *)(*iaru_fields[i].w))->value() ? "T" : "F";
+			iaru_fields[i].f_data = val;
+		}
 	}
 }
 
@@ -237,7 +267,7 @@ void iaru_clear_form()
 		else if (iaru_fields[i].w_type == 't')
 			((Fl_Input2 *)(*iaru_fields[i].w))->value("");
 		else if (iaru_fields[i].w_type == 's')
-			((Fl_Choice *)(*iaru_fields[i].w))->value(0);
+			((Fl_ListBox *)(*iaru_fields[i].w))->index(0);
 		else if (iaru_fields[i].w_type == 'e')
 			((FTextEdit *)(*iaru_fields[i].w))->clear();
 		else if (iaru_fields[i].w_type == 'b')
@@ -253,7 +283,7 @@ void iaru_update_form()
 		else if (iaru_fields[i].w_type == 't')
 			((Fl_Input2 *)(*iaru_fields[i].w))->value(iaru_fields[i].f_data.c_str());
 		else if (iaru_fields[i].w_type == 's')
-			((Fl_Choice *)(*iaru_fields[i].w))->value(atoi(iaru_fields[i].f_data.c_str()));
+			((Fl_ListBox *)(*iaru_fields[i].w))->index(atoi(iaru_fields[i].f_data.c_str()));
 		else if (iaru_fields[i].w_type == 'e') {
 			((FTextEdit *)(*iaru_fields[i].w))->clear();
 			((FTextEdit *)(*iaru_fields[i].w))->add(iaru_fields[i].f_data.c_str());
@@ -262,30 +292,36 @@ void iaru_update_form()
 	}
 }
 
-void iaru_make_buffer()
+void iaru_make_buffer(bool compress = false)
 {
-	iaru_update_fields();
-
+	string mbuff;
+	mbuff.clear();
 	for (int i = 0; i < iaru_num_fields; i++)
-		iaru_buffer.append( lineout( iaru_fields[i].f_type, iaru_fields[i].f_data ) );
+		mbuff.append( lineout( iaru_fields[i].f_type, iaru_fields[i].f_data ) );
+	if (compress) compress_maybe(mbuff);
+	iaru_buffer.append(mbuff);
 }
 
 void iaru_read_buffer(string data)
 {
-	bool data_ok = false;
 	clear_fields();
 	read_header(data);
 
-	for (int i = 0; i < iaru_num_fields; i++) {
+	for (int i = 0; i < iaru_num_fields; i++)
 		iaru_fields[i].f_data = findstr(data, iaru_fields[i].f_type);
-		if (!iaru_fields[i].f_data.empty()) data_ok = true;
-	}
 
 	iaru_update_form();
 }
 
 void iaru_cb_new()
 {
+	if (iaru_check_fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 1) {
+			update_header(CHANGED);
+			iaru_cb_save();
+		}
+	}
+	iaru_update_fields();
 	iaru_clear_form();
 	clear_header();
 	iaru_def_filename = ICS_msg_dir;
@@ -314,10 +350,33 @@ void iaru_cb_wrap_import(string wrapfilename, string inpbuffer)
 	iaru_using_template = false;
 }
 
+int eval_iaru_fsize()
+{
+	Ccrc16 chksum;
+	evalstr.assign("[WRAP:beg][WRAP:lf][WRAP:fn ");
+	evalstr.append(iaru_base_filename).append("]");
+	iaru_update_fields();
+	update_header(FROM);
+	evalstr.append(header("<iaru>"));
+	iaru_buffer.clear();
+	iaru_make_buffer(true);
+	if (iaru_buffer.empty()) return 0;
+	evalstr.append( iaru_buffer );
+	evalstr.append("[WRAP:chksum ").append(chksum.scrc16(evalstr)).append("][WRAP:end]");
+	return evalstr.length();
+}
+
 void iaru_cb_wrap_export()
 {
+	if (iaru_check_fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	iaru_update_fields();
+
 	if (iaru_base_filename == "new"IARU_FILE_EXT || iaru_base_filename == "default"IARU_FILE_EXT)
-		iaru_cb_save_as();
+		if (!iaru_cb_save_as()) return;
 
 	string wrapfilename = WRAP_send_dir;
 	wrapfilename.append(iaru_base_filename);
@@ -329,26 +388,38 @@ void iaru_cb_wrap_export()
 	if (p) {
 		string pext = fl_filename_ext(p);
 		wrapfilename = p;
-		update_header(true);
-		buffer.assign(header("<radiogram>", true, true));
-		iaru_make_buffer();
+
+		update_header(FROM);
+		iaru_buffer.assign(header("<iaru>"));
+		iaru_make_buffer(true);
 		export_wrapfile(iaru_base_filename, wrapfilename, iaru_buffer, pext != WRAP_EXT);
+
+		iaru_buffer.assign(header("<iaru>"));
+		iaru_make_buffer(false);
+		iaru_write(iaru_def_filename);
 	}
 }
 
 void iaru_cb_wrap_autosend()
 {
-	if (iaru_base_filename == "new"IARU_FILE_EXT || 
-		iaru_base_filename == "default"IARU_FILE_EXT ||
-		iaru_using_template == true)
-		iaru_cb_save_as();
+	if (iaru_check_fields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	iaru_update_fields();
 
-	string wrapfilename = WRAP_auto_dir;
-	wrapfilename.append("wrap_auto_file");
-	update_header(true);
-	buffer.assign(header("<radiogram>", true, true));
-	iaru_make_buffer();
-	export_wrapfile(iaru_base_filename, wrapfilename, iaru_buffer, false);
+	if (iaru_base_filename == "new"IARU_FILE_EXT || iaru_base_filename == "default"IARU_FILE_EXT)
+		if (!iaru_cb_save_as()) return;
+
+	update_header(FROM);
+	iaru_buffer.assign(header("<iaru>"));
+	iaru_make_buffer(true);
+	xfr_via_socket(iaru_base_filename, iaru_buffer);
+
+	iaru_buffer.assign(header("<iaru>"));
+	iaru_make_buffer(false);
+	iaru_write(iaru_def_filename);
 }
 
 void iaru_cb_load_template()
@@ -379,7 +450,10 @@ void iaru_cb_save_template()
 			"Template file\t*"IARU_TEMP_EXT,
 			iaru_def_filename.c_str());
 	if (p) {
-		clear_header();
+		update_header(CHANGED);
+		iaru_update_fields();
+		iaru_buffer.assign(header("<iaru>"));
+		iaru_make_buffer();
 		iaru_write(p);
 	}
 }
@@ -396,8 +470,14 @@ void iaru_cb_save_as_template()
 		iaru_def_template_name = p;
 		if (strlen(pext) == 0) iaru_def_template_name.append(IARU_TEMP_EXT);
 		remove_spaces_from_filename(iaru_def_template_name);
+
 		clear_header();
+		update_header(CHANGED);
+		iaru_update_fields();
+		iaru_buffer.assign(header("<iaru>"));
+		iaru_make_buffer();
 		iaru_write(iaru_def_template_name);
+
 		show_filename(iaru_def_template_name);
 		iaru_using_template = true;
 	}
@@ -420,29 +500,30 @@ void iaru_write(string s)
 {
 	FILE *iaru_file = fopen(s.c_str(), "w");
 	if (!iaru_file) return;
-	update_header();
-	buffer.assign(save_header("<radiogram>"));
-	iaru_make_buffer();
+
 	fwrite(iaru_buffer.c_str(), iaru_buffer.length(), 1, iaru_file);
 	fclose(iaru_file);
 }
 
-void iaru_cb_save_as()
+bool iaru_cb_save_as()
 {
 	const char *p;
 	string newfilename;
-	string name = named_file();
 
+	string name = named_file();
 	if (!name.empty()) {
 		name.append(IARU_FILE_EXT);
 		newfilename = ICS_msg_dir;
 		newfilename.append(name);
 	} else
 		newfilename = iaru_def_filename;
+
 	p = FSEL::saveas(_("Save data file"), "iaru\t*"IARU_FILE_EXT,
 						newfilename.c_str());
-	if (!p) return;
-	if (strlen(p) == 0) return;
+
+	if (!p) return false;
+	if (strlen(p) == 0) return false;
+
 	if (progStatus.rgnbr_fname) {
 		int n = atoi(progStatus.rgnbr.c_str());
 		n++;
@@ -465,11 +546,16 @@ void iaru_cb_save_as()
 	if (strlen(pext) == 0) iaru_def_filename.append(IARU_FILE_EXT);
 
 	remove_spaces_from_filename(iaru_def_filename);
-	clear_header();
+
+	update_header(NEW);
+	iaru_update_fields();
+	iaru_buffer.assign(header("<iaru>"));
+	iaru_make_buffer();
 	iaru_write(iaru_def_filename);
 
 	iaru_using_template = false;
 	show_filename(iaru_def_filename);
+	return true;
 }
 
 void iaru_cb_save()
@@ -480,7 +566,13 @@ void iaru_cb_save()
 		iaru_cb_save_as();
 		return;
 	}
+
+	if (iaru_check_fields()) update_header(CHANGED);
+	iaru_update_fields();
+	iaru_buffer.assign(header("<iaru>"));
+	iaru_make_buffer();
 	iaru_write(iaru_def_filename);
+
 	iaru_using_template = false;
 }
 
