@@ -1,6 +1,28 @@
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
+//
+// This file is part of flmsg
+//
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
+//
+// flrig is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #ifndef FLMSG_DIALOG_H
 #define FLMSG_DIALOG_H
 
+#include <string>
+
 #include <FL/Fl.H>
 #include <FL/Fl_Pixmap.H>
 #include <FL/Fl_Double_Window.H>
@@ -13,7 +35,6 @@
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Scroll.H>
-#include <FL/Fl_Choice.H>
 #include <FL/Fl_Browser.H>
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Input.H>
@@ -23,10 +44,10 @@
 
 #include "FTextView.h"
 #include "flinput2.h"
+#include "combo.h"
 #include "calendar.h"
 #include "flmsg.h"
 
-
 extern Fl_Output	*txt_filename;
 extern Fl_Input		*drop_box;
 extern Fl_Input		*drop_file;
@@ -67,10 +88,10 @@ extern Fl_Return_Button	*btnCloseOptions;
 
 extern Fl_Double_Window* arl_dialog();
 extern Fl_Browser	*select_arl;
-extern Fl_Input		*txt_arl_fill1;
-extern Fl_Input		*txt_arl_fill2;
-extern Fl_Input		*txt_arl_fill3;
-extern Fl_Input		*txt_arl_fill4;
+extern Fl_Input2	*txt_arl_fill1;
+extern Fl_Input2	*txt_arl_fill2;
+extern Fl_Input2	*txt_arl_fill3;
+extern Fl_Input2	*txt_arl_fill4;
 extern Fl_Button	*btn_arl_cancel;
 extern Fl_Button	*btn_arl_add;
 extern FTextEdit	*arl_text;
@@ -88,42 +109,62 @@ extern Fl_Round_Button	*btn_utc_format3;
 extern Fl_Round_Button	*btn_utc_format4;
 extern Fl_Round_Button	*btn_utc_format5;
 extern Fl_Button		*btn_close_date_time_dialog;
+extern Fl_Check_Button *btn_caplocal;
 
 extern Fl_Double_Window* personal_dialog();
-extern Fl_Input		*txt_my_call;
-extern Fl_Input		*txt_my_tel;
-extern Fl_Input		*txt_my_name;
-extern Fl_Input		*txt_my_addr;
-extern Fl_Input		*txt_my_city;
+extern Fl_Input2	*txt_my_call;
+extern Fl_Input2	*txt_my_tel;
+extern Fl_Input2	*txt_my_name;
+extern Fl_Input2	*txt_my_addr;
+extern Fl_Input2	*txt_my_city;
 
 extern Fl_Double_Window* headers_dialog();
 extern Fl_Input2*	txt_hdr_from;
 extern Fl_Input2*	txt_hdr_edit;
 
+extern Fl_Input2*	txt_socket_addr;
+extern Fl_Input2*	txt_socket_port;
 
 extern Fl_Double_Window* radiogram_dialog();
 extern Fl_Spinner	*cnt_wpl;
 extern Fl_Check_Button	*btn_rgnbr_fname;
-extern Fl_Input		*txt_rgnbr;
+extern Fl_Input2	*txt_rgnbr;
 
 extern Fl_Double_Window* config_files_dialog();
 extern Fl_Check_Button	*btn_open_on_export;
+extern Fl_Check_Button	*btn_use_compression;
 extern Fl_Check_Button	*btn_call_fname;
 extern Fl_Check_Button	*btn_dt_fname;
 extern Fl_Check_Button	*btn_sernbr_fname;
-extern Fl_Input		*txt_sernbr;
+extern Fl_Input2	*txt_sernbr;
 extern Fl_Input2	*txt_mars_roster_file;
 
 extern Fl_Double_Window* hx_dialog();
-extern Fl_Choice	*sel_hx_select;
+extern Fl_ListBox	*sel_hx_select;
 extern Fl_Button	*btn_hx_select_add;
 extern Fl_Input2	*txt_hx_select_text;
 extern Fl_Button	*btn_hx_select_cancel;
 extern Fl_Return_Button	*btn_hx_select_ok;
 extern Fl_Output	*txt_hx_instructions;
 
+extern Fl_Double_Window* socket_dialog();
+extern Fl_Input2 * txt_socket_addr;
+extern Fl_Input2 * txt_socket_port;
+extern Fl_Button * btn_close_socket_dialog;
+
+
 extern void select_form(int form);
 
+extern Fl_Output *txt_xfr_size_time;
+extern int transfer_size;
+
+extern Fl_ListBox	*cbo_modes;
+
+extern void clear_estimate();
+extern void estimate();
+extern bool valid_mode_check(string &md);
+extern void update_cbo_modes(string &fldigi_modes);
+
 //======================================================================
 // ICS
 //======================================================================
@@ -153,8 +194,8 @@ extern Fl_Input2	*txt_203_incident_info_officer;
 extern Fl_Input2	*txt_203_liaison_officer;
 extern Fl_Input2	*txt_203_prepared_by;
 extern Fl_Group		*tab_203_agency;
-extern Fl_Input2	*txt_203_agency[7];
-extern Fl_Input2	*txt_203_agency_name[7];
+extern Fl_Input2	*txt_203_agency[];
+extern Fl_Input2	*txt_203_agency_name[];
 extern Fl_Group		*tab_203_planning;
 extern Fl_Input2	*txt_203_planning_chief;
 extern Fl_Input2	*txt_203_planning_deputy;
@@ -162,7 +203,7 @@ extern Fl_Input2	*txt_203_resources_unit;
 extern Fl_Input2	*txt_203_situation_unit;
 extern Fl_Input2	*txt_203_documentation_unit;
 extern Fl_Input2	*txt_203_demobilization_unit;
-extern Fl_Input2	*txt_203_tech_spc[3];
+extern Fl_Input2	*txt_203_tech_spc[];
 extern Fl_Group		*tab_203_logistics;
 extern Fl_Input2	*txt_203_logistics_chief;
 extern Fl_Input2	*txt_203_logistics_deputy;
@@ -179,18 +220,18 @@ extern Fl_Input2	*txt_203_ops_chief;
 extern Fl_Input2	*txt_203_ops_deputy;
 extern Fl_Input2	*txt_203_b1_director;
 extern Fl_Input2	*txt_203_b1_deputy;
-extern Fl_Input2	*txt_203_b1_div[5];
-extern Fl_Input2	*txt_203_b1_grp[5];
+extern Fl_Input2	*txt_203_b1_div[];
+extern Fl_Input2	*txt_203_b1_grp[];
 extern Fl_Group		*tab_203_opsB;
 extern Fl_Input2	*txt_203_b2_director;
 extern Fl_Input2	*txt_203_b2_deputy;
-extern Fl_Input2	*txt_203_b2_div[5];
-extern Fl_Input2	*txt_203_b2_grp[5];
+extern Fl_Input2	*txt_203_b2_div[];
+extern Fl_Input2	*txt_203_b2_grp[];
 extern Fl_Group		*tab_203_opsC;
 extern Fl_Input2	*txt_203_b3_director;
 extern Fl_Input2	*txt_203_b3_deputy;
-extern Fl_Input2	*txt_203_b3_div[5];
-extern Fl_Input2	*txt_203_b3_grp[5];
+extern Fl_Input2	*txt_203_b3_div[];
+extern Fl_Input2	*txt_203_b3_grp[];
 extern Fl_Group		*tab_203_opsD;
 extern Fl_Input2	*txt_203_air_ops_br_dir;
 extern Fl_Input2	*txt_203_air_tactical_group_sup;
@@ -211,6 +252,9 @@ extern Fl_Input2	*txt_203_cost_unit;
 extern void create_ics205_tab();
 
 extern Fl_Group		*tab_ics205;
+extern Fl_Tabs		*tabs_ics205;
+extern Fl_Group		*tab_205_1;
+extern Fl_Group		*tab_205_2;
 extern Fl_Input2	*txt_205_name;
 extern Fl_Input2	*txt_205_dt_prepared;
 extern Fl_Button	*btn_205_dt_prepared;
@@ -218,13 +262,21 @@ extern Fl_Input2	*txt_205_dt_op_from;
 extern Fl_Button	*btn_205_dt_op_from;
 extern Fl_Input2	*txt_205_dt_op_to;
 extern Fl_Button	*btn_205_dt_op_to;
-extern Fl_Input2	*txt_205_type[8];
-extern Fl_Input2	*txt_205_channel[8];
-extern Fl_Input2	*txt_205_function[8];
-extern Fl_Input2	*txt_205_freqtone[8];
-extern Fl_Input2	*txt_205_assignment[8];
-extern Fl_Input2	*txt_205_remarks[8];
+extern Fl_Input2	*txt_205_func[];
+extern Fl_Input2	*txt_205_channel[];
+extern Fl_Input2	*txt_205_assign[];
+extern Fl_Input2	*txt_205_rx_freq[];
+extern Fl_Input2	*txt_205_rx_tone[];
+extern Fl_Input2	*txt_205_tx_freq[];
+extern Fl_Input2	*txt_205_tx_tone[];
+extern Fl_Input2	*txt_205_mode_adm[];
+extern Fl_Input2	*txt_205_remarks[];
 extern Fl_Input2	*txt_205_preparer;
+extern Fl_Input2	*txt_205_county;
+extern Fl_Input2	*txt_205_state;
+extern Fl_Input2	*txt_205_latitude;
+extern Fl_Input2	*txt_205_longitude;
+
 
 //======================================================================
 // ICS-205A
@@ -248,10 +300,10 @@ extern Fl_Button	*btn_205a_time_to;
 extern Fl_Input2	*txt_205a_prepared_by;
 extern Fl_Input2	*txt_205a_preparer_position;
 extern Fl_Input2	*txt_205a_preparer_date_time;
-extern Fl_Input2	*txt_205a_comm_group[32];
-extern Fl_Input2	*txt_205a_comm_position[32];
-extern Fl_Input2	*txt_205a_comm_name[32];
-extern Fl_Input2	*txt_205a_comm_info[32];
+extern Fl_Input2	*txt_205a_comm_group[];
+extern Fl_Input2	*txt_205a_comm_position[];
+extern Fl_Input2	*txt_205a_comm_name[];
+extern Fl_Input2	*txt_205a_comm_info[];
 
 //======================================================================
 // ICS-206
@@ -267,28 +319,28 @@ extern Fl_Input2	*txt_206_time_prepared;
 extern Fl_Button	*btn_206_time;
 extern Fl_DateInput	*txt_206_date_prepared;
 extern Fl_Button	*btn_206_date;
-extern Fl_Input2	*txt_206_medaid_sta[5];
-extern Fl_Input2	*txt_206_medaid_loc[5];
-extern Fl_Check_Button	*btn_206_medaid_paramedics[5];
+extern Fl_Input2	*txt_206_medaid_sta[];
+extern Fl_Input2	*txt_206_medaid_loc[];
+extern Fl_Check_Button	*btn_206_medaid_paramedics[];
 extern Fl_Input2	*txt_206_preparer;
 extern Fl_Input2	*txt_206_reviewer;
 extern Fl_Group		*tab_206_transport;
-extern Fl_Input2	*txt_206_transport_name[5];
-extern Fl_Input2	*txt_206_transport_address[5];
-extern Fl_Check_Button	*btn_206_transport_paramedics[5];
-extern Fl_Input2	*txt_206_transport_phone[5];
+extern Fl_Input2	*txt_206_transport_name[];
+extern Fl_Input2	*txt_206_transport_address[];
+extern Fl_Check_Button	*btn_206_transport_paramedics[];
+extern Fl_Input2	*txt_206_transport_phone[];
 extern Fl_Group		*tab_206_ambulance;
-extern Fl_Input2	*txt_206_ambulance_name[5];
-extern Fl_Input2	*txt_206_ambulance_loc[5];
-extern Fl_Check_Button	*btn_206_ambulance_paramedics[5];
+extern Fl_Input2	*txt_206_ambulance_name[];
+extern Fl_Input2	*txt_206_ambulance_loc[];
+extern Fl_Check_Button	*btn_206_ambulance_paramedics[];
 extern Fl_Group		*tab_206_hospital;
-extern Fl_Input2	*txt_206_hosp_name[5];
-extern Fl_Input2	*txt_206_hosp_address[5];
-extern Fl_Input2	*txt_206_hosp_phone[5];
-extern Fl_Input2	*txt_206_hosp_airtime[5];
-extern Fl_Input2	*txt_206_hosp_gndtime[5];
-extern Fl_Check_Button	*btn_206_hosp_helipad[5];
-extern Fl_Check_Button	*btn_206_hosp_burn_center[5];
+extern Fl_Input2	*txt_206_hosp_name[];
+extern Fl_Input2	*txt_206_hosp_address[];
+extern Fl_Input2	*txt_206_hosp_phone[];
+extern Fl_Input2	*txt_206_hosp_airtime[];
+extern Fl_Input2	*txt_206_hosp_gndtime[];
+extern Fl_Check_Button	*btn_206_hosp_helipad[];
+extern Fl_Check_Button	*btn_206_hosp_burn_center[];
 extern Fl_Group		*tab_206_med_proc;
 extern FTextEdit	*txt_206_procedure;
 
@@ -339,12 +391,12 @@ extern Fl_Input2	*txt_214_unit_name;
 extern Fl_Input2	*txt_214_unit_leader;
 extern Fl_Input2	*txt_214_prepared_by;
 extern Fl_Group		*tab_214_2;
-extern Fl_Input2	*txt_214_roster_name[16];
-extern Fl_Input2	*txt_214_roster_position[16];
-extern Fl_Input2	*txt_214_roster_home_base[16];
+extern Fl_Input2	*txt_214_roster_name[];
+extern Fl_Input2	*txt_214_roster_position[];
+extern Fl_Input2	*txt_214_roster_home_base[];
 extern Fl_Group		*tab_214_3;
-extern Fl_Input2	*txt_214_activity_time[16];
-extern Fl_Input2	*txt_214_activity_event[16];
+extern Fl_Input2	*txt_214_activity_time[];
+extern Fl_Input2	*txt_214_activity_event[];
 
 //======================================================================
 // ICS-216
@@ -370,12 +422,41 @@ extern Fl_Input2	*txt_216_op_period;
 extern Fl_Input2	*txt_216_tac_freq;
 extern Fl_Input2	*txt_216_prepared_by;
 extern Fl_Group		*tab_216_2;
-extern Fl_Input2	*txt_216_div_grp[4];
-extern Fl_Input2	*txt_216_agency[4];
+extern Fl_Input2	*txt_216_div_grp[];
+extern Fl_Input2	*txt_216_agency[];
 extern Fl_Group		*tab_216_3;
-extern Fl_Input2	*txt_216_ag[36];
-extern Fl_Input2	*txt_216_id[36];
-extern Fl_Input2	*txt_216_rr[36];
+extern Fl_Input2	*txt_216_ag[];
+extern Fl_Input2	*txt_216_id[];
+extern Fl_Input2	*txt_216_rr[];
+
+//======================================================================
+// ICS-309
+//======================================================================
+extern void create_ics309_tab();
+
+extern Fl_Group	*tab_ics309;
+extern Fl_Tabs		*tab_ics309_type;
+extern Fl_Group	*tab_309_1;
+extern Fl_Input2	*txt_309_incident;
+extern Fl_Input2	*txt_309_date_fm;
+extern Fl_Button	*btn_309_date_fm;
+extern Fl_Input2	*txt_309_date_to;
+extern Fl_Button	*btn_309_date_to;
+extern Fl_Input2	*txt_309_time_fm;
+extern Fl_Button	*btn_309_time_fm;
+extern Fl_Input2	*txt_309_time_to;
+extern Fl_Button	*btn_309_time_to;
+
+extern Fl_Input2	*txt_309_prepared_by;
+extern Fl_Input2	*txt_309_radio_net;
+extern Fl_Input2	*txt_309_radio_operator;
+extern Fl_Input2	*txt_309_preparer_date_time;
+
+extern Fl_Group	*tab_309_2;
+extern Fl_Input2	*txt_309_comm_time[];
+extern Fl_Input2	*txt_309_comm_from[];
+extern Fl_Input2	*txt_309_comm_to[];
+extern Fl_Input2	*txt_309_comm_msg[];
 
 //======================================================================
 // HICS
@@ -585,7 +666,7 @@ extern Fl_Tabs		*tabs_iaru;
 extern Fl_Group		*tab_iaru_message;
 extern Fl_Group		*tab_iaru_information;
 extern Fl_Input2	*iaru_txt_nbr;
-extern Fl_Choice	*iaru_sel_prec;
+extern Fl_ListBox	*iaru_sel_prec;
 extern Fl_Input2	*iaru_txt_station;
 extern Fl_Input2	*iaru_txt_orig;
 extern Fl_Input2	*iaru_txt_t1;
@@ -617,7 +698,7 @@ extern Fl_Group		*tab_radiogram;
 extern Fl_Tabs		*tabs_radiogram;
 extern Fl_Group		*tab_radiogram_message;
 extern Fl_Input2	*txt_rg_nbr;
-extern Fl_Choice	*sel_rg_prec;
+extern Fl_ListBox	*sel_rg_prec;
 extern Fl_Input2	*txt_rg_hx;
 extern Fl_Button	*btn_rg_hx;
 extern Fl_Input2	*txt_rg_station;
@@ -652,6 +733,8 @@ extern Fl_Input2	*txt_rg_snt_net;
 extern Fl_Input2	*txt_rg_dt5;
 extern Fl_Button	*btn_rgDateTime5;
 
+extern Fl_Check_Button	*btn_rg_standard;
+
 //======================================================================
 // Plaintext
 //======================================================================
@@ -676,6 +759,14 @@ extern void			create_blank_tab();
 extern Fl_Group		*tab_blank;
 extern FTextEdit	*txt_blank_msg;
 
+//======================================================================
+// CSV
+//======================================================================
+extern void			create_csv_tab();
+
+extern Fl_Group		*tab_csv;
+extern FTextEdit	*txt_csv_msg;
+
 //======================================================================
 // Drag-n-Drop
 //======================================================================
@@ -719,7 +810,7 @@ extern Fl_Input2	*txt_mars_daily_incident;
 extern Fl_Input2	*txt_mars_daily_de;
 extern Fl_Button	*btn_mars_daily_de_me;
 extern Fl_Input2	*txt_mars_daily_nbr;
-extern Fl_Choice	*sel_mars_daily_prec;
+extern Fl_ListBox	*sel_mars_daily_prec;
 extern Fl_Input2	*txt_mars_daily_dtg;
 extern Fl_Button	*btn_mars_daily_dtg;
 extern Fl_Input2	*txt_mars_daily_fm_name;
@@ -748,7 +839,7 @@ extern Fl_Input2	*txt_mars_daily_sitrep_loc;
 extern Fl_Input2	*txt_mars_daily_dtg_24hr;
 extern Fl_Button	*btn_mars_daily_dtg_24hr;
 extern Fl_Input2	*txt_mars_daily_netcall;
-extern Fl_Choice	*sel_mars_daily_e_x;
+extern Fl_ListBox	*sel_mars_daily_e_x;
 extern Fl_Input2	*txt_mars_daily_dtg_active;
 extern Fl_Button	*btn_mars_daily_dtg_active;
 extern Fl_Input2	*txt_mars_daily_dtg_deactive;
@@ -772,7 +863,7 @@ extern Fl_Group		*tab_mars_ineei_1;
 
 extern Fl_Input2	*txt_mars_ineei_DE;
 extern Fl_Input2	*txt_mars_ineei_NBR;
-extern Fl_Choice	*sel_mars_ineei_PREC;
+extern Fl_ListBox	*sel_mars_ineei_PREC;
 extern Fl_Input2	*txt_mars_ineei_DTG;
 extern Fl_Input2	*txt_mars_ineei_FMNAME;
 extern Fl_Input2	*txt_mars_ineei_FMCALL;
@@ -786,6 +877,30 @@ extern Fl_Input2	*txt_mars_ineei_INFO1STATE;
 extern Fl_Input2	*txt_mars_ineei_INFO2POS;
 extern Fl_Input2	*txt_mars_ineei_INFO2CALL;
 extern Fl_Input2	*txt_mars_ineei_INFO2STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO3POS;
+extern Fl_Input2	*txt_mars_ineei_INFO3CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO3STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO4POS;
+extern Fl_Input2	*txt_mars_ineei_INFO4CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO4STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO5POS;
+extern Fl_Input2	*txt_mars_ineei_INFO5CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO5STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO6POS;
+extern Fl_Input2	*txt_mars_ineei_INFO6CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO6STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO7POS;
+extern Fl_Input2	*txt_mars_ineei_INFO7CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO7STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO8POS;
+extern Fl_Input2	*txt_mars_ineei_INFO8CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO8STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO9POS;
+extern Fl_Input2	*txt_mars_ineei_INFO9CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO9STATE;
+extern Fl_Input2	*txt_mars_ineei_INFO10POS;
+extern Fl_Input2	*txt_mars_ineei_INFO10CALL;
+extern Fl_Input2	*txt_mars_ineei_INFO10STATE;
 extern Fl_Input2	*txt_mars_ineei_INCIDENT;
  
 extern Fl_Group		*tab_mars_ineei_2;
@@ -818,7 +933,7 @@ extern Fl_Group		*tab_mars_net_2;
 
 extern Fl_Input2	*txt_mars_net_DE;
 extern Fl_Input2	*txt_mars_net_NBR;
-extern Fl_Choice	*sel_mars_net_PREC;
+extern Fl_ListBox	*sel_mars_net_PREC;
 extern Fl_Input2	*txt_mars_net_DTG;
 extern Fl_Input2	*txt_mars_net_FMNAME;
 extern Fl_Input2	*txt_mars_net_FMCALL;
@@ -869,7 +984,7 @@ extern Fl_Button	*btn_mars_army_pick_fm;
 extern Fl_Button	*btn_mars_army_pick_to;
 extern Fl_Button	*btn_mars_army_pick_info;
 
-extern Fl_Choice	*sel_mars_army_prec;
+extern Fl_ListBox	*sel_mars_army_prec;
 
 //======================================================================
 // MARS Navy Standard Message
@@ -880,8 +995,11 @@ extern void			create_mars_navy();
 extern Fl_Group		*tab_mars_navy;
 
 extern Fl_Tabs		*tab_mars_navy_type;
-extern Fl_Group		*tab_mars_navy_1;
-extern Fl_Group		*tab_mars_navy_2;
+extern Fl_Group	*tab_mars_navy_1;
+extern Fl_Group	*tab_mars_navy_2;
+
+extern Fl_Input2	*txt_mars_navy_de;
+extern Fl_Input2	*txt_mars_navy_serno;
 
 extern Fl_Input2	*txt_mars_navy_dtg;
 extern Fl_Input2	*txt_mars_navy_fm;
@@ -896,7 +1014,140 @@ extern Fl_Button	*btn_mars_navy_pick_fm;
 extern Fl_Button	*btn_mars_navy_pick_to;
 extern Fl_Button	*btn_mars_navy_pick_info;
 
-extern Fl_Choice	*sel_mars_navy_prec;
+extern Fl_ListBox	*sel_mars_navy_prec;
+
+//======================================================================
+// National Hurricane Center, Hurricane Weather Report Form
+//======================================================================
+
+extern void create_wxhc_tab();
+
+extern Fl_Group	*tab_wxhc;
+extern Fl_Tabs		*tab_wxhc_type;
+
+extern Fl_Group	*tab_wxhc_1;
+
+extern Fl_Input2	*w_wxhc_rptsta;
+extern Fl_Input2	*w_wxhc_email;
+extern Fl_Input2	*w_wxhc_phone;
+extern Fl_Input2	*w_wxhc_addr;
+extern Fl_Input2	*w_wxhc_city;
+extern Fl_Input2	*w_wxhc_state;
+extern Fl_Input2	*w_wxhc_country;
+extern Fl_Input2	*w_wxhc_lat;
+extern Fl_Input2	*w_wxhc_long;
+extern Fl_Input2	*w_wxhc_date;
+extern Fl_Input2	*w_wxhc_time;
+extern Fl_Check_Button *w_wxhc_meas;
+extern Fl_Check_Button *w_wxhc_est;
+extern Fl_Input2	*w_wxhc_wind_speed;
+extern Fl_ListBox	*w_wxhc_wind_speed_units;
+extern Fl_Input2	*w_wxhc_wind_gusts;
+extern Fl_ListBox	*w_wxhc_wind_gusts_units;
+extern Fl_ListBox	*w_wxhc_wind_dir;
+extern Fl_Input2	*w_wxhc_wind_degrees;
+extern Fl_Input2	*w_wxhc_baro_press;
+extern Fl_ListBox	*w_wxhc_baro_units;
+
+extern Fl_Group	*tab_wxhc_2;
+
+extern FTextEdit	*w_wxhc_comments;
+
+//======================================================================
+// Severe Weather Report Form
+//======================================================================
+
+extern void create_severe_wx_tab();
+
+extern Fl_Group	*tab_severe_wx;
+extern Fl_Tabs		*tab_severe_wx_type;
+
+extern Fl_Group	*tab_severe_wx_1;
+extern Fl_DateInput	*w_severe_wx_date;
+extern Fl_Input2	*w_severe_wx_time;
+extern Fl_ListBox	*w_severe_wx_zone;
+extern Fl_Check_Button *w_severe_wx_meas;
+extern Fl_Check_Button *w_severe_wx_est;
+extern Fl_ListBox	*w_severe_wx_state;
+extern Fl_ListBox	*w_severe_wx_county;
+extern Fl_Input2	*w_severe_wx_city;
+extern Fl_Input2	*w_severe_wx_wind_speed;
+extern Fl_ListBox	*w_severe_wx_hail_size;
+extern Fl_Check_Button *w_severe_wx_exact;
+extern Fl_Check_Button *w_severe_wx_not_exact;
+extern Fl_Check_Button *w_severe_wx_tornado;
+extern Fl_Check_Button *w_severe_wx_funnel_cloud;
+extern Fl_Check_Button *w_severe_wx_wall_cloud;
+extern Fl_Check_Button *w_severe_wx_hail;
+extern Fl_Check_Button *w_severe_wx_high_wind;
+extern Fl_Check_Button *w_severe_wx_measured;
+extern Fl_Check_Button *w_severe_wx_estimated;
+extern Fl_Check_Button *w_severe_wx_flood;
+extern Fl_Check_Button *w_severe_wx_flash_flood;
+extern Fl_Check_Button *w_severe_wx_other;
+extern Fl_Check_Button *w_severe_wx_dmg_yes;
+extern Fl_Check_Button *w_severe_wx_dmg_no;
+extern Fl_Check_Button *w_severe_wx_inj_yes;
+extern Fl_Check_Button *w_severe_wx_inj_no;
+
+extern Fl_Group	*tab_severe_wx_2;
+extern FTextEdit	*w_severe_wx_narrative;
+
+//======================================================================
+// Severe Weather Report Form
+//======================================================================
+
+extern void create_storm_tab();
+
+extern Fl_Group		*tab_storm;
+extern Fl_Tabs			*tab_storm_type;
+
+extern Fl_Group		*tab_storm_1;
+
+extern Fl_DateInput	*w_storm_date;
+extern Fl_Input2		*w_storm_time;
+extern Fl_ListBox		*w_storm_zone;
+
+extern Fl_ListBox		*w_storm_state;
+extern Fl_ListBox		*w_storm_county;
+extern Fl_Input2		*w_storm_location;
+
+extern Fl_Check_Button *w_storm_tornado;
+extern Fl_ListBox		*w_storm_tornado_cat;
+
+extern Fl_Check_Button *w_storm_hail;
+extern Fl_ListBox		*w_storm_hail_size;
+
+extern Fl_Check_Button *w_storm_wind;
+extern Fl_ListBox		*w_storm_wind_cat;
+
+extern Fl_Check_Button *w_storm_flood;
+extern Fl_ListBox		*w_storm_flood_cat;
+
+extern Fl_Check_Button *w_storm_damage;
+extern Fl_ListBox		*w_storm_damage_cat;
+
+extern Fl_Check_Button *w_storm_snow;
+extern Fl_ListBox		*w_storm_snow_tot;
+extern Fl_ListBox		*w_storm_snow_dur;
+
+extern Fl_Check_Button *w_storm_f_rain;
+extern Fl_ListBox		*w_storm_f_rain_tot;
+extern Fl_ListBox		*w_storm_f_rain_dur;
+
+extern Fl_Check_Button *w_storm_h_rain;
+extern Fl_ListBox		*w_storm_h_rain_tot;
+extern Fl_ListBox		*w_storm_h_rain_dur;
+
+extern Fl_Input2		*w_storm_o_name;
+extern Fl_Input2		*w_storm_o_email;
+extern Fl_Input2		*w_storm_o_tele;
+extern Fl_ListBox		*w_storm_o_profile;
+
+extern Fl_Group		*tab_storm_2;
+extern FTextEdit		*w_storm_details;
+
+extern void set_storm_counties(int);
 
 //======================================================================
 // Red Cross
@@ -1125,5 +1376,81 @@ extern Fl_Input2	*rdx_5739B_bs;
 extern Fl_Input2	*rdx_5739B_gi;
 
 //======================================================================
+// Civil Air Patrol (CAP) forms
+//======================================================================
+// CAP-105, Radio Message
+
+extern Fl_Group	*tab_cap105;
+extern Fl_Tabs		*tabs_cap105;
+extern Fl_Group	*tab_cap105_message;
+extern Fl_Group	*tab_cap105_information;
+
+extern Fl_Input2 *cap105_txt_nbr;
+extern Fl_ListBox *cap105_sel_prec;
+
+extern Fl_Input2 *cap105_txt_dtm;
+extern Fl_Button *cap105_btn_dtm;
+
+extern Fl_Input2 *cap105_txt_from;
+extern Fl_Input2 *cap105_txt_to;
+extern Fl_Input2 *cap105_txt_info;
+
+extern Fl_Input2 *cap105_txt_subj;
+
+extern Fl_Input2 *cap105_txt_grpcnt;
+extern Fl_Button *cap105_btn_check;
+
+extern FTextEdit *cap105_txt_msg;
+
+extern Fl_Input2 *cap105_txt_rcvd_fm;
+extern Fl_Input2 *cap105_txt_rcvd_dtm;
+extern Fl_Button *cap105_btn_rcvd_dtm;
+
+extern Fl_Input2 *cap105_txt_sent_to;
+extern Fl_Input2 *cap105_txt_sent_dtm;
+extern Fl_Button *cap105_btn_sent_dtm;
+
+extern Fl_Input2 *cap105_txt_rcvop;
+extern Fl_Input2 *cap105_txt_sendop;
+
+extern void create_cap105_tab();
+
+// cap110 comms report
+
+extern Fl_Group	*tab_cap110;
+extern Fl_Tabs		*tab_cap110_type;
+extern Fl_Group	*tab_cap110_1;
+
+extern Fl_Input2	*txt_110_mission_nbr;
+extern Fl_Input2	*txt_110_station;
+extern Fl_Input2	*txt_110_date;
+extern Fl_Button	*btn_110_date;
+extern Fl_Input2	*txt_110_ds_a;
+extern Fl_Input2	*txt_110_ds_b;
+extern Fl_Input2	*txt_110_ds_c;
+extern Fl_Input2	*txt_110_ds_d;
+extern Fl_Input2	*txt_110_ds_e;
+extern Fl_Input2	*txt_110_ds_f;
+
+extern Fl_Group	*tab_cap110_2;
+extern Fl_Input2	*txt_110_comm_time[23];
+extern Fl_Input2	*txt_110_comm_call[23];
+extern Fl_Input2	*txt_110_comm_chref[23];
+extern Fl_Input2	*txt_110_comm_rem[23];
+
+extern void create_cap110_tab();
+
+//======================================================================
+// external string arrays
+//
+extern const std::string counties;
+extern const std::string states[];
+extern int numstates;
+extern const std::string hail_size[];
+extern int num_hail_size;
+extern const string tzones[];
+extern int num_tzones;
+extern const std::string us_tzones[];
+extern int num_us_tzones;
 
 #endif
