@@ -1,26 +1,31 @@
-// =====================================================================
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// templates.h
+// This file is part of flmsg
 //
-// Author: Dave Freese, W1HKJ
-// Copyright: 2010
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  It is
-// copyright under the GNU General Public License.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with the program; if not, write to the Free Software
-// Foundation, Inc.
-// 59 Temple Place, Suite 330
-// Boston, MA  02111-1307 USA
-//
-// =====================================================================
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #ifndef _TEMPLATES_H
 #define _TEMPLATES_H
 
+extern const char cap105_html_template[];
+extern const char cap105_txt_template[];
+extern const char cap110_html_template[];
+extern const char cap110_text_template[];
+
 extern const char ics203_html_template[];
 extern const char ics203_text_template[];
 extern const char ics205_html_template[];
@@ -35,6 +40,8 @@ extern const char ics214_html_template[];
 extern const char ics214_text_template[];
 extern const char ics216_html_template[];
 extern const char ics216_text_template[];
+extern const char ics309_html_template[];
+extern const char ics309_text_template[];
 
 extern const char hics203_html_template[];
 extern const char hics203_text_template[];
@@ -59,6 +66,9 @@ extern const char pt_txt_template[];
 extern const char blank_html_template[];
 extern const char blank_txt_template[];
 
+extern const char csv_html_template[];
+extern const char csv_txt_template[];
+
 extern const char baddata_html_template[];
 
 extern const char mars_daily_html_template[];
@@ -86,5 +96,14 @@ extern const char redx_5739A_text_template[];
 extern const char redx_5739B_html_template[];
 extern const char redx_5739B_text_template[];
 
+extern const char wxhc_html_template[]; 
+extern const char wxhc_text_template[];
+
+extern const char severe_wx_html_template[];
+extern const char severe_wx_text_template[];
+
+extern const char storm_html_template[];
+extern const char storm_text_template[];
+
 #endif
 
