@@ -1,22 +1,22 @@
-// =====================================================================
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// radiogram.cxx
+// This file is part of flmsg
 //
-// Author: Dave Freese, W1HKJ
-// Copyright: 2010
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  It is
-// copyright under the GNU General Public License.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with the program; if not, write to the Free Software
-// Foundation, Inc.
-// 59 Temple Place, Suite 330
-// Boston, MA  02111-1307 USA
-//
-// =====================================================================
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <stdlib.h>
 #include <iostream>
@@ -172,33 +172,35 @@ string _rg_rcv_fm	= ":rfm:";		// 21
 string _rg_rcv_net	= ":rnt:";		// 22
 string _rg_dt5		= ":dt5:";		// 23
 string _rg_svc		= ":svc:";		// 24
+string _rg_standard	= ":std:";		// 25
 
 FIELD rgfields[] = {
-{ _rg_nbr,		"", (void **)&txt_rg_nbr,	't' },	// 0
-{ _rg_prec,		"0", (void **)&sel_rg_prec,	's' },	// 1
+{ _rg_nbr,		"", (void **)&txt_rg_nbr,		't' },	// 0
+{ _rg_prec,		"0", (void **)&sel_rg_prec,		's' },	// 1
 { _rg_hx,		"", (void **)&txt_rg_hx,		't' },	// 2
 { _rg_d1,		"", (void **)&txt_rg_d1,		't' },	// 3
 { _rg_t1,		"", (void **)&txt_rg_t1,		't' },	// 4
-{ _rg_dt2,		"", (void **)&txt_rg_dt2,	't' },	// 5
-{ _rg_dt3,		"", (void **)&txt_rg_dt3,	't' },	// 6
+{ _rg_dt2,		"", (void **)&txt_rg_dt2,		't' },	// 5
+{ _rg_dt3,		"", (void **)&txt_rg_dt3,		't' },	// 6
 { _rg_to,		"", (void **)&txt_rg_to,		't' },	// 7
-{ _rg_phone,	"", (void **)&txt_rg_phone,	't' },	// 8
+{ _rg_phone,	"", (void **)&txt_rg_phone,		't' },	// 8
 { _rg_opnote,	"", (void **)&txt_rg_opnote,	't' },	// 9
-{ _rg_msg,		"", (void **)&txt_rg_msg,	'e' },	// 10
-{ _rg_sig,		"", (void **)&txt_rg_sig,	't' },	// 11
-{ _rg_opnote2,	"", (void **)&txt_rg_opnote2,'t' },	// 12
-{ _rg_check,	"", (void **)&txt_rg_check,	't' },	// 13
-{ _rg_station,	"", (void **)&txt_rg_station,'t' },	// 14
-{ _rg_place,	"", (void **)&txt_rg_place,	't' },	// 15
-{ _rg_orig,		"", (void **)&txt_rg_orig,	't' },	// 16
-{ _rg_dlvd_to,	"", (void **)&txt_rg_dlvd_to,'t' },	// 17
-{ _rg_sent_to,	"", (void **)&txt_rg_sent_to,'t' },	// 18
-{ _rg_snt_net,	"", (void **)&txt_rg_snt_net,'t' },	// 19
-{ _rg_dt4,		"", (void **)&txt_rg_dt4,	't' },	// 20
+{ _rg_msg,		"", (void **)&txt_rg_msg,		'e' },	// 10
+{ _rg_sig,		"", (void **)&txt_rg_sig,		't' },	// 11
+{ _rg_opnote2,	"", (void **)&txt_rg_opnote2,	't' },	// 12
+{ _rg_check,	"", (void **)&txt_rg_check,		't' },	// 13
+{ _rg_station,	"", (void **)&txt_rg_station,	't' },	// 14
+{ _rg_place,	"", (void **)&txt_rg_place,		't' },	// 15
+{ _rg_orig,		"", (void **)&txt_rg_orig,		't' },	// 16
+{ _rg_dlvd_to,	"", (void **)&txt_rg_dlvd_to,	't' },	// 17
+{ _rg_sent_to,	"", (void **)&txt_rg_sent_to,	't' },	// 18
+{ _rg_snt_net,	"", (void **)&txt_rg_snt_net,	't' },	// 19
+{ _rg_dt4,		"", (void **)&txt_rg_dt4,		't' },	// 20
 { _rg_rcv_fm,	"", (void **)&txt_rg_rcv_fm,	't' },	// 21
-{ _rg_rcv_net,	"", (void **)&txt_rg_rcv_net,'t' },	// 22
-{ _rg_dt5,		"", (void **)&txt_rg_dt5,	't' },	// 23
-{ _rg_svc,		"", (void **)&btn_rg_svc,	'b' }	// 24
+{ _rg_rcv_net,	"", (void **)&txt_rg_rcv_net,	't' },	// 22
+{ _rg_dt5,		"", (void **)&txt_rg_dt5,		't' },	// 23
+{ _rg_svc,		"F", (void **)&btn_rg_svc,		'b' },	// 24
+{ _rg_standard,	"T", (void **)&btn_rg_standard,	'B' }	// 25
 };
 
 bool using_rg_template = false;
@@ -261,21 +263,32 @@ static char valid_input[] = "0123456789/ ABCDEFGHIJKLMNOPQRSTUVWXYZ\n";
 void cb_rg_filter_input(Fl_Widget *wdg)
 {
 	Fl_Input2 *inp = (Fl_Input2 *)wdg;
+	int p = inp->position();
 	string s = inp->value();
 	ucase(s);
 	for (size_t n = 0; n < s.length(); n++)
 		if (strchr(valid_input, s[n]) == NULL)
 			s.erase(n,1);
 	inp->value(s.c_str());
+	inp->position(p);
 }
 
 void clear_rgfields()
 {
 	for (int i = 0; i < num_rgfields; i++) {
-		if (rgfields[i].w_type == 's')
-			rgfields[i].f_data = "0";
-		else
-			rgfields[i].f_data.clear();
+		switch (rgfields[i].w_type) {
+			case 's': 
+				rgfields[i].f_data = "0";
+				break;
+			case 'b':
+				rgfields[i].f_data = "F";
+				break;
+			case 'B':
+				rgfields[i].f_data = "T";
+				break;
+			default:
+				rgfields[i].f_data.clear();
+		}
 	}
 }
 
@@ -289,7 +302,38 @@ string numeric(int n)
 void set_rg_choices() {
 	sel_rg_prec->clear();
 	sel_rg_prec->add(precitems);
-	sel_rg_prec->value(0);
+	sel_rg_prec->index(0);
+}
+
+bool check_rgfields()
+{
+	string temp;
+	for (int i = 0; i < num_rgfields; i++) {
+		if (rgfields[i].w == NULL) return false;
+		if (rgfields[i].w_type == 'd') {
+			if (rgfields[i].f_data != ((Fl_DateInput *)(*rgfields[i].w))->value())
+				return true;
+		} else if (rgfields[i].w_type == 't') {
+			if (rgfields[i].f_data != ((Fl_Input2 *)(*rgfields[i].w))->value())
+				return true;
+		} else if (rgfields[i].w_type == 's') {
+			int choice = ((Fl_ListBox *)(*rgfields[i].w))->index();
+			if (rgfields[i].f_data != numeric(choice))
+				return true;
+		} else if (rgfields[i].w_type == 'e') {
+			if (rgfields[i].f_data != ((FTextEdit *)(*rgfields[i].w))->buffer()->text())
+				return true;
+		} else if (rgfields[i].w_type == 'b') {
+			temp = ((Fl_Button *)(*rgfields[i].w))->value() ? "T" : "F";
+			if (rgfields[i].f_data != temp)
+				return true;
+		} else if (rgfields[i].w_type == 'B'){
+			temp = ((Fl_Button *)(*rgfields[i].w))->value() ? "T" : "F";
+			if (rgfields[i].f_data != temp)
+				return true;
+		}
+	}
+	return false;
 }
 
 void update_rgfields()
@@ -300,13 +344,15 @@ void update_rgfields()
 		else if (rgfields[i].w_type == 't')
 			rgfields[i].f_data = ((Fl_Input2 *)(*rgfields[i].w))->value();
 		else if (rgfields[i].w_type == 's') {
-			int choice = ((Fl_Choice *)(*rgfields[i].w))->value();
+			int choice = ((Fl_ListBox *)(*rgfields[i].w))->index();
 			if (choice >= 0) 
 				rgfields[i].f_data = numeric(choice);
 		} else if (rgfields[i].w_type == 'e')
 			rgfields[i].f_data = ((FTextEdit *)(*rgfields[i].w))->buffer()->text();
 		else if (rgfields[i].w_type == 'b')
 			rgfields[i].f_data = ((Fl_Button *)(*rgfields[i].w))->value() ? "T" : "F";
+		else if (rgfields[i].w_type == 'B')
+			rgfields[i].f_data = ((Fl_Button *)(*rgfields[i].w))->value() ? "T" : "F";
 	}
 }
 
@@ -325,11 +371,13 @@ void clear_rg_form()
 		else if (rgfields[i].w_type == 't')
 			((Fl_Input2 *)(*rgfields[i].w))->value("");
 		else if (rgfields[i].w_type == 's')
-			((Fl_Choice *)(*rgfields[i].w))->value(0);
+			((Fl_ListBox *)(*rgfields[i].w))->index(0);
 		else if (rgfields[i].w_type == 'e')
 			((FTextEdit *)(*rgfields[i].w))->clear();
 		else if (rgfields[i].w_type == 'b')
 			((Fl_Button *)(*rgfields[i].w))->value(0);
+		else if (rgfields[i].w_type == 'B')
+			((Fl_Button *)(*rgfields[i].w))->value(1);
 	update_rgfields();
 }
 
@@ -341,21 +389,25 @@ void update_rg_form()
 		else if (rgfields[i].w_type == 't')
 			((Fl_Input2 *)(*rgfields[i].w))->value(rgfields[i].f_data.c_str());
 		else if (rgfields[i].w_type == 's')
-			((Fl_Choice *)(*rgfields[i].w))->value(atoi(rgfields[i].f_data.c_str()));
+			((Fl_ListBox *)(*rgfields[i].w))->index(atoi(rgfields[i].f_data.c_str()));
 		else if (rgfields[i].w_type == 'e') {
 			((FTextEdit *)(*rgfields[i].w))->clear();
 			((FTextEdit *)(*rgfields[i].w))->add(rgfields[i].f_data.c_str());
 		} else if (rgfields[i].w_type == 'b')
 			((Fl_Button *)(*rgfields[i].w))->value(rgfields[i].f_data == "T" ? 1 : 0);
+		else if (rgfields[i].w_type == 'B')
+			((Fl_Button *)(*rgfields[i].w))->value(rgfields[i].f_data == "T" ? 1 : 0);
 	}
 }
 
-void make_rg_buffer()
+void make_rg_buffer(bool compress = false)
 {
-	update_rgfields();
-
+	string mbuff;
+	mbuff.clear();
 	for (int i = 0; i < num_rgfields; i++)
-		buffer.append( lineout( rgfields[i].f_type, rgfields[i].f_data ) );
+		mbuff.append( lineout( rgfields[i].f_type, rgfields[i].f_data ) );
+	if (compress) compress_maybe(mbuff);
+	buffer.append(mbuff);
 }
 
 void read_rg_buffer(string data)
@@ -364,8 +416,10 @@ void read_rg_buffer(string data)
 	clear_fields();
 	read_header(data);
 
+	string temp;
 	for (int i = 0; i < num_rgfields; i++) {
-		rgfields[i].f_data = findstr(data, rgfields[i].f_type);
+		temp = findstr(data, rgfields[i].f_type);
+		if (!temp.empty()) rgfields[i].f_data = temp;
 		if (!rgfields[i].f_data.empty()) data_ok = true;
 	}
 	if (!data_ok)
@@ -377,6 +431,12 @@ void read_rg_buffer(string data)
 
 void cb_rg_new()
 {
+	if (check_rgfields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 1) {
+			update_header(CHANGED);
+			cb_rg_save();
+		}
+	}
 	clear_rg_form();
 	clear_header();
 	def_rg_filename = ICS_msg_dir;
@@ -427,10 +487,36 @@ void cb_rg_wrap_import(string wrapfilename, string inpbuffer)
 	using_rg_template = false;
 }
 
+int eval_rg_fsize()
+{
+	Ccrc16 chksum;
+	evalstr.assign("[WRAP:beg][WRAP:lf][WRAP:fn ");
+	evalstr.append(base_rg_filename).append("]");
+	update_rgfields();
+	update_header(FROM);
+	evalstr.append(header("<radiogram>"));
+	buffer.clear();
+	make_rg_buffer(true);
+	if (buffer.empty()) return 0;
+	evalstr.append( buffer );
+	evalstr.append("[WRAP:chksum ").append(chksum.scrc16(evalstr)).append("][WRAP:end]");
+	return evalstr.length();
+}
+
 void cb_rg_wrap_export()
 {
+	if (btn_rg_check->labelcolor() == FL_RED)
+		cb_rg_check();
+
+	if (check_rgfields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	update_rgfields();
+
 	if (base_rg_filename == "new"RGFILE_EXT || base_rg_filename == "default"RGFILE_EXT)
-		cb_rg_save_as();
+		if (!cb_rg_save_as()) return;
 
 	string wrapfilename = WRAP_send_dir;
 	wrapfilename.append(base_rg_filename);
@@ -444,28 +530,41 @@ void cb_rg_wrap_export()
 			cb_rg_check();
 		string pext = fl_filename_ext(p);
 		wrapfilename = p;
-		update_header(true);
-		buffer.assign(header("<radiogram>", true, true));
-		make_rg_buffer();
+
+		update_header(FROM);
+		buffer.assign(header("<radiogram>"));
+		make_rg_buffer(true);
 		export_wrapfile(base_rg_filename, wrapfilename, buffer, pext != WRAP_EXT);
+
+		buffer.assign(header("<radiogram>"));
+		make_rg_buffer(false);
+		write_rg(def_rg_filename);
 	}
 }
 
 void cb_rg_wrap_autosend()
 {
-	if (base_rg_filename == "new"RGFILE_EXT || 
-		base_rg_filename == "default"RGFILE_EXT ||
-		using_rg_template == true)
-		cb_rg_save_as();
-
-	string wrapfilename = WRAP_auto_dir;
-	wrapfilename.append("wrap_auto_file");
 	if (btn_rg_check->labelcolor() == FL_RED)
 		cb_rg_check();
-	update_header(true);
-	buffer.assign(header("<radiogram>", true, true));
-	make_rg_buffer();
-	export_wrapfile(base_rg_filename, wrapfilename, buffer, false);
+
+	if (check_rgfields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	update_rgfields();
+
+	if (base_rg_filename == "new"RGFILE_EXT || base_rg_filename == "default"RGFILE_EXT)
+		if (!cb_rg_save_as()) return;
+
+	update_header(FROM);
+	buffer.assign(header("<radiogram>"));
+	make_rg_buffer(true);
+	xfr_via_socket(base_rg_filename, buffer);
+
+	buffer.assign(header("<radiogram>"));
+	make_rg_buffer(false);
+	write_rg(def_rg_filename);
 }
 
 void cb_rg_load_template()
@@ -496,7 +595,10 @@ void cb_rg_save_template()
 			"Template file\t*"RGTEMP_EXT,
 			def_rg_filename.c_str());
 	if (p) {
-		clear_header();
+		update_header(CHANGED);
+		update_rgfields();
+		buffer.assign(header("<radiogram>"));
+		make_rg_buffer();
 		write_rg(p);
 	}
 }
@@ -513,8 +615,13 @@ void cb_rg_save_as_template()
 		def_rg_TemplateName = p;
 		if (strlen(pext) == 0) def_rg_TemplateName.append(RGTEMP_EXT);
 		remove_spaces_from_filename(def_rg_TemplateName);
+
 		clear_header();
+		update_header(CHANGED);
+		buffer.assign(header("<radiogram>"));
+		make_rg_buffer();
 		write_rg(def_rg_TemplateName);
+
 		show_filename(def_rg_TemplateName);
 		using_rg_template = true;
 	}
@@ -537,14 +644,12 @@ void write_rg(string s)
 {
 	FILE *rgfile = fopen(s.c_str(), "w");
 	if (!rgfile) return;
-	update_header();
-	buffer.assign(save_header("<radiogram>"));
-	make_rg_buffer();
+
 	fwrite(buffer.c_str(), buffer.length(), 1, rgfile);
 	fclose(rgfile);
 }
 
-void cb_rg_save_as()
+bool cb_rg_save_as()
 {
 	const char *p;
 	string newfilename;
@@ -558,8 +663,10 @@ void cb_rg_save_as()
 		newfilename = def_rg_filename;
 	p = FSEL::saveas(_("Save data file"), "radiogram\t*"RGFILE_EXT,
 						newfilename.c_str());
-	if (!p) return;
-	if (strlen(p) == 0) return;
+
+	if (!p) return false;
+	if (strlen(p) == 0) return false;
+
 	if (progStatus.rgnbr_fname) {
 		int n = atoi(progStatus.rgnbr.c_str());
 		n++;
@@ -582,11 +689,16 @@ void cb_rg_save_as()
 	if (strlen(pext) == 0) def_rg_filename.append(RGFILE_EXT);
 
 	remove_spaces_from_filename(def_rg_filename);
-	clear_header();
+
+	update_header(NEW);
+	update_rgfields();
+	buffer.assign(header("<radiogram>"));
+	make_rg_buffer();
 	write_rg(def_rg_filename);
 
 	using_rg_template = false;
 	show_filename(def_rg_filename);
+	return true;
 }
 
 void cb_rg_save()
@@ -597,6 +709,11 @@ void cb_rg_save()
 		cb_rg_save_as();
 		return;
 	}
+
+	if (check_rgfields()) update_header(CHANGED);
+	update_rgfields();
+	buffer.assign(header("<radiogram>"));
+	make_rg_buffer();
 	write_rg(def_rg_filename);
 	using_rg_template = false;
 }
@@ -625,42 +742,44 @@ void cb_rg_check()
 		return;
 	}
 
-	// convert to uppercase
-	for (size_t n = 0; n < temp.length(); n++)
-		temp[n] = toupper(temp[n]);
-
 	size_t pos = string::npos;
 
-	strip_lfs(temp);
-	// remove trailing period
-	if (temp[temp.length()-1] == '.') temp.erase(temp.length()-1,1);
-	// convert punctuation
-	for (int n = 0; punctuation[n]; n += 2)
-		while ((pos = temp.find(punctuation[n])) != string::npos)
-			temp.replace(pos,1,punctuation[n+1]);
-	//convert embedded periods
-	while ((pos = temp.find(".")) != string::npos)
-		if (isdigit(temp[pos-1]) || isdigit(temp[pos+1]))
-			temp[pos] = 'R';
-		else
-			temp.replace(pos, 1, " DOT ");
+	if (btn_rg_standard->value()) {
+// convert to uppercase
+		for (size_t n = 0; n < temp.length(); n++)
+			temp[n] = toupper(temp[n]);
 
-	// remove any user inserted end-of-lines
+		strip_lfs(temp);
+// remove trailing period
+		if (temp[temp.length()-1] == '.') temp.erase(temp.length()-1,1);
+		// convert punctuation
+		for (int n = 0; punctuation[n]; n += 2)
+			while ((pos = temp.find(punctuation[n])) != string::npos)
+				temp.replace(pos, strlen(punctuation[n]), punctuation[n+1]);
+//convert embedded periods
+		while ((pos = temp.find(".")) != string::npos)
+			if (isdigit(temp[pos-1]) || isdigit(temp[pos+1]))
+				temp[pos] = 'R';
+			else
+				temp.replace(pos, 1, " DOT ");
+	}
+
+// remove any user inserted end-of-lines
 	while ((pos = temp.find('\n')) != string::npos) temp[pos] = ' ';
 
-	// only single spaces no trailing spaces, no leading spaces
+// only single spaces no trailing spaces, no leading spaces
 	while ((pos = temp.find("  ")) != string::npos) temp.erase(pos,1);
 	while (temp[temp.length() -1] == ' ') temp.erase(temp.length()-1, 1);
 	if (temp[0] == ' ') temp.erase(0,1);
 
-	// count number of words in textdef_rg_filename
+// count number of words in textdef_rg_filename
 	int numwords = 1;
 	if (temp.length()) {
 		pos = 0;
 		while ((pos = temp.find(" ", pos + 1)) != string::npos) numwords++;
 	}
 
-	// no more than 5 words to a line
+// no more than specified # words to a line
 	if (numwords > progStatus.wpl) {
 		int wc = numwords;
 		size_t pos = 0;
@@ -705,7 +824,7 @@ void cb_rg_html()
 			if (nbr >= 0 && nbr < (sizeof(s_prec) / sizeof(*s_prec)))
 				html_text = s_prec[nbr];
 			else
-				html_text = "";
+				html_text = s_prec[0];
 			replacestr( form, rgfields[i].f_type, html_text );
 		} else if (rgfields[i].w_type == 'b') {
 			replacestr( form, rgfields[i].f_type, rgfields[i].f_data == "T" ? yes : no);
@@ -753,7 +872,7 @@ void cb_rg_html_fcopy()
 			if (nbr >= 0 && nbr < (sizeof(s_prec) / sizeof(*s_prec)))
 				html_text = s_prec[nbr];
 			else
-				html_text = "";
+				html_text = s_prec[0];
 			replacestr( form, rgfields[i].f_type, html_text);
 		} else if (rgfields[i].w_type == 'b') {
 			replacestr( form, rgfields[i].f_type, rgfields[i].f_data == "T" ? yes : no);
@@ -781,7 +900,7 @@ void cb_rg_textout()
 	string rgname;
 	string lines;
 	string str;
-	int nbr = 0;//, fn;
+	size_t nbr = 0;
 	rgname = ICS_dir;
 	rgname.append("radiogram.txt");
 
@@ -794,6 +913,8 @@ void cb_rg_textout()
 		str.clear();
 		if (rgfields[i].f_type == _rg_prec) {
 			sscanf(rgfields[i].f_data.c_str(), "%d", &nbr);
+			if (nbr < 0) nbr = 0;
+			if (nbr >= sizeof(s_prec)/sizeof(*s_prec)) nbr = 0;
 			str = s_prec[nbr];
 			if (str.find("TEST") != string::npos) {				// test message
 				if (str.find("EMERGENCY") == string::npos)
