@@ -1,22 +1,22 @@
-// =====================================================================
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// redx_5739AA.cxx
+// This file is part of flmsg
 //
-// Author: Dave Freese, W1HKJ
-// Copyright: 2010
+// flrig is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  It is
-// copyright under the GNU General Public License.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with the program; if not, write to the Free Software
-// Foundation, Inc.
-// 59 Temple Place, Suite 330
-// Boston, MA  02111-1307 USA
-//
-// =====================================================================
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <stdlib.h>
 #include <iostream>
@@ -218,6 +218,61 @@ void clear_redx_5739Afields()
 	clearQUAD_5739A(redx_QUAD_5739A);
 }
 
+bool checkQUAD_5739A(QUAD_5739A *p)
+{
+	int i = 0;
+	float f = 0;
+	char c = ' ';
+	QTYPE_5739A qt = E;
+	while ((qt = p->qtype) != E) {
+		if (p->widget == NULL) return false;
+		switch (qt) {
+			case B:
+				if (*((bool *)(p->ptr)) != ((Fl_Check_Button *)p->widget)->value())
+					return true;;
+				break;
+			case S:
+				if (*((string *)(p->ptr)) != ((Fl_Input2 *)p->widget)->value())
+					return true;
+				break;
+			case T:
+				if (*((string *)(p->ptr)) != ((FTextEdit *)p->widget)->buffer()->text())
+					return true;
+				break;
+			case C:
+				c = ' ';
+				if (((Fl_Input2 *)p->widget)->value()[0])
+					c = ((Fl_Input2 *)p->widget)->value()[0];
+				if (*((char *)(p->ptr)) != c)
+					return true;
+				break;
+			case I:
+				i = 0;
+				if (((Fl_Input2 *)p->widget)->value()[0])
+					sscanf( ((Fl_Input2 *)p->widget)->value(), "%d", &i);
+				if (*((int *)(p->ptr)) != i)
+					return true;
+				break;
+			case F:
+				f = 0;
+				if (((Fl_Input2 *)p->widget)->value()[0])
+					sscanf( ((Fl_Input2 *)p->widget)->value(), "%f", &f);
+				if (*((float *)(p->ptr)) != f)
+					return true;
+				break;
+			case E:
+			default: return false;
+		}
+		p++;
+	}
+	return false;
+}
+
+bool check_redx_5739Afields()
+{
+	return checkQUAD_5739A(redx_QUAD_5739A);
+}
+
 void updateQUAD_5739A(QUAD_5739A *p)
 {
 	int i = 0;
@@ -318,6 +373,8 @@ void clear_redx_5739A_form()
 	update_redx_5739Aform();
 }
 
+static string mbuff;
+
 void make_buffQUAD_5739A(QUAD_5739A *p)
 {
 	string one = "1"; string zero = "0";
@@ -328,34 +385,34 @@ void make_buffQUAD_5739A(QUAD_5739A *p)
 		switch (qt) {
 			case B:
 				if (*((bool *)(p->ptr)) == true)
-					buffredx_5739A.append( lineout( p->htmnbr, *((bool *)(p->ptr)) ? one : zero));
+					mbuff.append( lineout( p->htmnbr, *((bool *)(p->ptr)) ? one : zero));
 				break;
 			case S:
 				if (((string *)(p->ptr))->length())
-					buffredx_5739A.append( lineout( p->htmnbr, *((string *)(p->ptr))));
+					mbuff.append( lineout( p->htmnbr, *((string *)(p->ptr))));
 				break;
 			case T:
-				buffredx_5739A.append( lineout( p->htmnbr, *((string *)(p->ptr))));
+				mbuff.append( lineout( p->htmnbr, *((string *)(p->ptr))));
 				break;
 			case C:
 				if ((*(char *)(p->ptr)) != 0 && *((char *)(p->ptr)) != ' ') {
 					sval = " ";
 					sval[0] = *((char *)(p->ptr));
-					buffredx_5739A.append( lineout( p->htmnbr, sval));
+					mbuff.append( lineout( p->htmnbr, sval));
 				}
 				break;
 			case I:
 				if (*((int*)(p->ptr)) > 0) {
 					snprintf(szval, sizeof(szval), "%d", *((int *)(p->ptr)) );
 					sval = szval;
-					buffredx_5739A.append( lineout( p->htmnbr, sval) );
+					mbuff.append( lineout( p->htmnbr, sval) );
 				}
 				break;
 			case F:
 				if (*((float *)(p->ptr)) > 0) {
 					snprintf(szval, sizeof(szval), "%f", *((float *)(p->ptr)));
 					sval = szval;
-					buffredx_5739A.append( lineout( p->htmnbr, sval) );
+					mbuff.append( lineout( p->htmnbr, sval) );
 				}
 				break;
 			case E:
@@ -365,11 +422,12 @@ void make_buffQUAD_5739A(QUAD_5739A *p)
 	}
 }
 
-void make_buffredx_5739A()
+void make_buffredx_5739A(bool compress = false)
 {
-	update_redx_5739Afields();
-
+	mbuff.clear();
 	make_buffQUAD_5739A(redx_QUAD_5739A);
+	if (compress) compress_maybe(mbuff);
+	buffredx_5739A.append(mbuff);
 }
 
 void readQUAD_5739A(string data, QUAD_5739A *p)
@@ -419,6 +477,12 @@ void read_redx_5739A_buffer(string data)
 
 void cb_redx_5739A_new()
 {
+	if (check_redx_5739Afields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 1) {
+			update_header(CHANGED);
+			cb_redx_5739A_save();
+		}
+	}
 	clear_redx_5739A_form();
 	clear_header();
 	def_redx_5739A_filename = ICS_msg_dir;
@@ -447,10 +511,34 @@ void cb_redx_5739A_wrap_import(string wrapfilename, string inpbuffer)
 	using_redx_5739A_template = false;
 }
 
+int eval_redx_5739A_fsize()
+{
+	Ccrc16 chksum;
+	evalstr.assign("[WRAP:beg][WRAP:lf][WRAP:fn ");
+	evalstr.append(base_redx_5739A_filename).append("]");
+	update_redx_5739Afields();
+	update_header(FROM);
+	evalstr.append(header("<redx_5739A>"));
+	buffredx_5739A.clear();
+	make_buffredx_5739A(true);
+	if (buffredx_5739A.empty()) return 0;
+	compress_maybe( buffredx_5739A );
+	evalstr.append( buffredx_5739A );
+	evalstr.append("[WRAP:chksum ").append(chksum.scrc16(evalstr)).append("][WRAP:end]");
+	return evalstr.length();
+}
+
 void cb_redx_5739A_wrap_export()
 {
+	if (check_redx_5739Afields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	update_redx_5739Afields();
+
 	if (base_redx_5739A_filename == "new"FREDX5739A_EXT || base_redx_5739A_filename == "default"FREDX5739A_EXT)
-		cb_redx_5739A_save_as();
+		if (!cb_redx_5739A_save_as()) return;
 
 	string wrapfilename = WRAP_send_dir;
 	wrapfilename.append(base_redx_5739A_filename);
@@ -462,26 +550,38 @@ void cb_redx_5739A_wrap_export()
 	if (p) {
 		string pext = fl_filename_ext(p);
 		wrapfilename = p;
-		update_header(true);
-		buffredx_5739A.assign(header("<redx_5739A>", true, true));
-		make_buffredx_5739A();
+
+		update_header(FROM);
+		buffredx_5739A.assign(header("<redx_5739A>"));
+		make_buffredx_5739A(true);
 		export_wrapfile(base_redx_5739A_filename, wrapfilename, buffredx_5739A, pext != ".wrap");
+
+		buffredx_5739A.assign(header("<redx_5739A>"));
+		make_buffredx_5739A(false);
+		write_redx_5739A(def_redx_5739A_filename);
 	}
 }
 
 void cb_redx_5739A_wrap_autosend()
 {
-	if (base_redx_5739A_filename == "new"FREDX5739A_EXT || 
-		base_redx_5739A_filename == "default"FREDX5739A_EXT ||
-		using_redx_5739A_template == true)
-		cb_redx_5739A_save_as();
+	if (check_redx_5739Afields()) {
+		if (fl_choice2("Form modified, save?", "No", "Yes", NULL) == 0)
+			return;
+		update_header(CHANGED);
+	}
+	update_redx_5739Afields();
 
-	string wrapfilename = WRAP_auto_dir;
-	wrapfilename.append("wrap_auto_file");
-	update_header(true);
-	buffredx_5739A.assign(header("<redx_5739A>", true, true));
-	make_buffredx_5739A();
-	export_wrapfile(base_redx_5739A_filename, wrapfilename, buffredx_5739A, false);
+	if (base_redx_5739A_filename == "new"FREDX5739A_EXT || base_redx_5739A_filename == "default"FREDX5739A_EXT)
+		if (!cb_redx_5739A_save_as()) return;
+
+	update_header(FROM);
+	buffredx_5739A.assign(header("<redx_5739A>"));
+	make_buffredx_5739A(true);
+	xfr_via_socket(base_redx_5739A_filename, buffredx_5739A);
+
+	buffredx_5739A.assign(header("<redx_5739A>"));
+	make_buffredx_5739A(false);
+	write_redx_5739A(def_redx_5739A_filename);
 }
 
 void cb_redx_5739A_load_template()
@@ -512,7 +612,10 @@ void cb_redx_5739A_save_template()
 			"Template file\t*"TREDX5739A_EXT,
 			def_redx_5739A_filename.c_str());
 	if (p) {
-		clear_header();
+		update_header(CHANGED);
+		update_redx_5739Afields();
+		buffredx_5739A.assign(header("<redx_5739A>"));
+		make_buffredx_5739A();
 		write_redx_5739A(p);
 	}
 }
@@ -529,8 +632,14 @@ void cb_redx_5739A_save_as_template()
 		def_redx_5739A_TemplateName = p;
 		if (strlen(pext) == 0) def_redx_5739A_TemplateName.append(TREDX5739A_EXT);
 		remove_spaces_from_filename(def_redx_5739A_TemplateName);
+
 		clear_header();
+		update_header(CHANGED);
+		update_redx_5739Afields();
+		buffredx_5739A.assign(header("<redx_5739A>"));
+		make_buffredx_5739A();
 		write_redx_5739A(def_redx_5739A_TemplateName);
+
 		show_filename(def_redx_5739A_TemplateName);
 		using_redx_5739A_template = true;
 	}
@@ -553,17 +662,12 @@ void write_redx_5739A(string s)
 {
 	FILE *fileredx_5739A = fopen(s.c_str(), "w");
 	if (!fileredx_5739A) return;
-	update_header();
-	buffredx_5739A.assign(save_header("<redx_5739A>"));
-	make_buffredx_5739A();
-	fprintf(fileredx_5739A, "%s", buffredx_5739A.c_str());
+
+	fwrite(buffredx_5739A.c_str(), buffredx_5739A.length(), 1, fileredx_5739A);
 	fclose(fileredx_5739A);
-	FILE *TEST = fopen("test.txt", "w");
-	fwrite(buffredx_5739A.c_str(), buffredx_5739A.length(), 1, TEST);
-	fclose(TEST);
 }
 
-void cb_redx_5739A_save_as()
+bool cb_redx_5739A_save_as()
 {
 	const char *p;
 	string newfilename;
@@ -578,8 +682,10 @@ void cb_redx_5739A_save_as()
 
 	p = FSEL::saveas(_("Save data file"), "ICS-redx_5739A\t*"FREDX5739A_EXT,
 					newfilename.c_str());
-	if (!p) return;
-	if (strlen(p) == 0) return;
+
+	if (!p) return false;
+	if (strlen(p) == 0) return false;
+
 	if (progStatus.sernbr_fname) {
 		string haystack = p;
 		if (haystack.find(newfilename) != string::npos) {
@@ -598,11 +704,16 @@ void cb_redx_5739A_save_as()
 	if (strlen(pext) == 0) def_redx_5739A_filename.append(FREDX5739A_EXT);
 
 	remove_spaces_from_filename(def_redx_5739A_filename);
-	clear_header();
+
+	update_header(NEW);
+	update_redx_5739Afields();
+	buffredx_5739A.assign(header("<redx_5739A>"));
+	make_buffredx_5739A();
 	write_redx_5739A(def_redx_5739A_filename);
 
 	using_redx_5739A_template = false;
 	show_filename(def_redx_5739A_filename);
+	return true;
 }
 
 void cb_redx_5739A_save()
@@ -613,7 +724,13 @@ void cb_redx_5739A_save()
 		cb_redx_5739A_save_as();
 		return;
 	}
+
+	if (check_redx_5739Afields()) update_header(CHANGED);
+	update_redx_5739Afields();
+	buffredx_5739A.assign(header("<redx_5739A>"));
+	make_buffredx_5739A();
 	write_redx_5739A(def_redx_5739A_filename);
+
 	using_redx_5739A_template = false;
 }
 
