@@ -3,18 +3,18 @@
 //
 // Copyright (C) 2007-2009
 //              Stelios Bounanos, M0GLD
+// ----------------------------------------------------------------------------
+// Copyright (C) 2014
+//              David Freese, W1HKJ
 //
-// Copyright (C) 2008-2009
-//              Dave Freese, W1HKJ
+// This file is part of flmsg
 //
-// This file is part of fldigi.
-//
-// fldigi is free software; you can redistribute it and/or modify
+// flrig is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// flrig is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
@@ -91,6 +91,7 @@ FTextBase::FTextBase(int x, int y, int w, int h, const char *l)
 	 scrollbar_width((int)floor(scrollbar_width() * 7.0/8.0));
 
 	reset_styles(SET_FONT | SET_SIZE | SET_COLOR);
+	read_cb = NULL;
 }
 
 void FTextBase::clear()
@@ -303,9 +304,6 @@ void FTextBase::set_style(int attr, Fl_Font f, int s, Fl_Color c, int set)
 }
 
 /// Reads a file and inserts its contents.
-/// change all occurrences of ^ to ^^ to prevent get_tx_char from
-/// treating the carat as a control sequence, ie: ^r ^R ^t ^T ^L ^C
-/// get_tx_char passes ^^ as a single ^
 ///
 /// @return 0 on success, -1 on error
 int FTextBase::readFile(const char* fn)
@@ -331,11 +329,6 @@ int FTextBase::readFile(const char* fn)
 	if (pos == tbuf->length()) { // optimise for append
 		while (fgets(buf, sizeof(buf), tfile)) {
 			newbuf = buf;
-			p = 0;
-			while ((p = newbuf.find('^',p)) != string::npos) {
-				newbuf.insert(p, "^");
-				p += 2;
-			}
 			tbuf->append(newbuf.c_str());
 			memset(buf, 0, BUFSIZ+1);
 		}
@@ -363,6 +356,7 @@ int FTextBase::readFile(const char* fn)
 	insert_position(pos);
 	show_insert_position();
 
+	if (read_cb) (read_cb)(fn);
 	return ret;
 }
 
